@@ -1,52 +1,51 @@
-/*
- *Copyright (C) 2004-2008 Qball Cow <qball@sarine.nl>
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
+/* Gnome Music Player Client (GMPC)
+ * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Project homepage: http://gmpc.wikia.com/
+ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- */
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdlib.h>
 #include <string.h>
-#include <glade/glade.h>
 #include <config.h>
-#include <regex.h>
-
-#include "plugin.h"
 
 #include "main.h"
 #include "misc.h"
 #include "playlist3.h"
 #include "playlist3-current-playlist-browser.h"
-#include "config1.h"
 #include "TreeSearchWidget.h"
 #include "gmpc-mpddata-model.h"
 #include "gmpc-mpddata-model-playlist.h"
 #include "gmpc-mpddata-treeview.h"
 #include "eggcolumnchooserdialog.h"
+#include "advanced_search.h"
 #ifdef USE_SYSTEM_LIBSEXY
-#include <sexy-icon-entry.h>
-#else
-#include "sexy-icon-entry.h"
+#include <libsexy/sexy-icon-entry.h>
 #endif
 #include "playlist3-messages.h"
 
 #include "playlist3-playlist-editor.h"
 
 
+static void pl3_current_playlist_browser_paste_after_songs(GtkTreeView *tree, GList *paste_list);
+static void pl3_current_playlist_browser_paste_before_songs(GtkTreeView *tree, GList *paste_list);
+static void pl3_current_playlist_browser_delete_selected_songs (void);
+
+
 static void pl3_current_playlist_browser_scroll_to_current_song(void);
 static void pl3_current_playlist_browser_add(GtkWidget *cat_tree);
 
@@ -55,6 +54,7 @@ static void pl3_current_playlist_browser_unselected(GtkWidget *container);
 
 static int pl3_current_playlist_browser_cat_menu_popup(GtkWidget *menu, int type, GtkWidget *tree, GdkEventButton *event);
 
+static void pl3_current_playlist_browser_activate(void);
 
 static void pl3_current_playlist_status_changed(MpdObj *mi, ChangedStatusType what, void *userdata);
 static int pl3_current_playlist_browser_add_go_menu(GtkWidget *menu);
@@ -78,10 +78,6 @@ GtkTreeModel *playlist = NULL;
 GtkWidget *pl3_cp_tree = NULL;
 static gboolean search_keep_open = FALSE;
 
-static GQueue *cut_queue = NULL;
-
-static GRegex *search_regex = NULL;
-
 static void pl3_cp_current_song_changed(GmpcMpdDataModelPlaylist *model2,GtkTreePath *path, GtkTreeIter *iter,gpointer data)
 {
     GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(pl3_cp_tree));
@@ -131,10 +127,8 @@ static void pl3_total_playtime_changed(GmpcMpdDataModelPlaylist *model, unsigned
     }
 }
 
-static void pl3_cp_init()
+static void pl3_cp_init(void)
 {
-    GString *string = g_string_new("(");
-    int i;
     playlist = (GtkTreeModel *)gmpc_mpddata_model_playlist_new(gmpcconn,connection);
     gmpc_mpddata_model_disable_image(GMPC_MPDDATA_MODEL(playlist));
     pl3_current_playlist_browser_init();
@@ -142,16 +136,14 @@ static void pl3_cp_init()
     g_signal_connect(G_OBJECT(playlist), "total_playtime_changed", G_CALLBACK(pl3_total_playtime_changed), NULL);
 
 
-    cut_queue = g_queue_new();
-    for(i=0;i<MPD_TAG_NUM_OF_ITEM_TYPES;i++)
-    {
-        g_string_append(string, mpdTagItemKeys[i]);
-        if(i< (MPD_TAG_NUM_OF_ITEM_TYPES-1))
-            g_string_append(string,"|");
-    }
-    g_string_append(string, ")[ ]*[=:][ ]*");
-    search_regex = g_regex_new(string->str, G_REGEX_CASELESS, 0, NULL);
-    g_string_free(string, TRUE);
+    gmpc_easy_command_add_entry(gmpc_easy_command,
+                    _("switch play queue"),"",
+                    _("Switch to play queue"),
+                    (GmpcEasyCommandCallback *)pl3_current_playlist_browser_activate, NULL); 
+    gmpc_easy_command_add_entry(gmpc_easy_command,
+                    _("Clear play queue"),"",
+                    _("Clear play queue"),
+                    (GmpcEasyCommandCallback *)pl3_current_playlist_browser_clear_playlist, NULL); 
 }
 
 gmpcPlBrowserPlugin current_playlist_gbp = {
@@ -167,7 +159,7 @@ gmpcPlBrowserPlugin current_playlist_gbp = {
 gmpcPlugin current_playlist_plug = {
 	.name = 							"Play Queue Browser",
 	.version = 							{1,1,1},
-	.plugin_type =						GMPC_PLUGIN_PL_BROWSER,
+	.plugin_type =						GMPC_PLUGIN_PL_BROWSER|GMPC_INTERNALL,
     .init   =                           pl3_cp_init,
 	.browser = 							&current_playlist_gbp,
 	.mpd_status_changed = 				pl3_current_playlist_status_changed,
@@ -176,10 +168,6 @@ gmpcPlugin current_playlist_plug = {
 	.save_yourself = 					pl3_current_playlist_save_myself
 };
 
-
-/* external objects */
-extern GladeXML *pl3_xml;
-
 /* internal */
 
 static GtkWidget *pl3_cp_sw = NULL;
@@ -219,22 +207,10 @@ static void pl3_current_playlist_column_changed(GtkTreeView *tree)
 	g_list_free(cols);
 }
 
-void pl3_current_playlist_destroy()
+void pl3_current_playlist_destroy(void)
 {
 	if(pl3_cp_tree)
 	{
-		GList *iter,*cols = gtk_tree_view_get_columns(GTK_TREE_VIEW(pl3_cp_tree));
-		for(iter = cols; iter; iter = g_list_next(iter))
-		{
-			gpointer data = g_object_get_data(G_OBJECT(iter->data), "colid");
-			int colid = GPOINTER_TO_INT(data);
-			gchar *string = g_strdup_printf("%i", colid);
-			int width = gtk_tree_view_column_get_width(GTK_TREE_VIEW_COLUMN(iter->data));
-			cfg_set_single_value_as_int(config, "current-playlist-column-width", string,width);
-			q_free(string);
-		}
-		g_list_free(cols);
-
 		/* destroy the entry */ 
 		if(pl3_curb_tree_ref)
 		{
@@ -262,8 +238,6 @@ void pl3_current_playlist_destroy()
 		}
 		pl3_cp_tree =  NULL;
 	}
-    if(search_regex) g_regex_unref(search_regex);
-    search_regex = NULL;
 }
 
 
@@ -272,63 +246,29 @@ void pl3_current_playlist_destroy()
 static GtkTreeModel *mod_fill = NULL;
 static GtkWidget *filter_entry = NULL;
 static guint timeout=0;
-
+static int quick_search = 0;
 static gboolean mod_fill_do_entry_changed(GtkWidget *entry, GtkWidget *tree)
 {
     const gchar *text2 = gtk_entry_get_text(GTK_ENTRY(entry));
     
     if(strlen(text2) > 0)
     {
+    
         MpdData *data = NULL;
-        gchar **text = g_regex_split(search_regex, text2, 0);// g_strsplit(text2, "=", 0);
-        int i= 0;
-        int searched = 0;
-        for(i=0; text && text[i] ;i++)
-        {
-            int type = mpd_misc_get_tag_by_name(g_strstrip(text[i]));
-            if(type != MPD_TAG_NOT_FOUND && text[i+1])
-            {
-                gchar **split = tokenize_string(text[i+1]);
-                int j;
-                for(j=0;split && split[j];j++)
-                {
-                    if(!searched){
-                        mpd_playlist_search_start(connection, FALSE);
-                        searched = 1;
-                    }
-                    mpd_playlist_search_add_constraint(connection, type,split[j]);
-                }
-                if(split)g_strfreev(split);
-
-            }
-            else 
-            {
-                gchar **split = tokenize_string(text[i]);
-                int j;
-                for(j=0;split && split[j];j++)
-                {
-                    if(!searched){
-                        mpd_playlist_search_start(connection, FALSE);
-                        searched = 1;
-                    }
-                    mpd_playlist_search_add_constraint(connection,MPD_TAG_ITEM_ANY,split[j]);
-                }
-                if(split)g_strfreev(split);
-            }
-        }
-        if(text)g_strfreev(text);
-        if(searched)
-            data = mpd_playlist_search_commit(connection);
+        data = advanced_search(text2, TRUE);
         gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(mod_fill), data);
-
+        quick_search = TRUE;
         gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_cp_tree), mod_fill);
+        gtk_widget_show(entry);
     }
     else
     {
         gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_cp_tree), playlist);
+
         gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(mod_fill), NULL);
         if(!search_keep_open)
         {
+            quick_search = 0;
             gtk_widget_hide(entry);
             gtk_widget_grab_focus(pl3_cp_tree);
         }
@@ -347,7 +287,9 @@ static gboolean mod_fill_entry_key_press_event(GtkWidget *entry, GdkEventKey *ev
         {
             search_keep_open = FALSE;
             gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_cp_tree), playlist);
+
             gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(mod_fill), NULL);
+            quick_search = 0;
             gtk_widget_hide(entry);
             gtk_widget_grab_focus(pl3_cp_tree);
             return TRUE;
@@ -382,10 +324,19 @@ static void pl3_current_playlist_browser_init(void)
 	pl3_cp_vbox = gtk_vbox_new(FALSE,6);
     tree = gmpc_mpddata_treeview_new("current-pl", FALSE, GTK_TREE_MODEL(playlist));
 
+    g_signal_connect(G_OBJECT(tree), "paste_before", G_CALLBACK(pl3_current_playlist_browser_paste_before_songs), NULL);
+    g_signal_connect(G_OBJECT(tree), "paste_after", G_CALLBACK(pl3_current_playlist_browser_paste_after_songs), NULL);
+    g_signal_connect(G_OBJECT(tree), "cut", G_CALLBACK(pl3_current_playlist_browser_delete_selected_songs), NULL);
+
+
     /* filter */
     mod_fill = (GtkTreeModel *)gmpc_mpddata_model_new();
+#ifdef USE_SYSTEM_LIBSEXY
     entry = sexy_icon_entry_new(); 
     sexy_icon_entry_add_clear_button(SEXY_ICON_ENTRY(entry));
+#else
+    entry = gtk_entry_new();
+#endif    
     gtk_box_pack_start(GTK_BOX(pl3_cp_vbox), entry, FALSE, TRUE,0);
     filter_entry= entry;
     g_signal_connect(G_OBJECT(entry), "changed", G_CALLBACK(mod_fill_entry_changed), tree);
@@ -405,7 +356,6 @@ static void pl3_current_playlist_browser_init(void)
     gmpc_mpddata_treeview_enable_click_fix(GMPC_MPDDATA_TREEVIEW(tree));
     /* setup signals */
 	g_signal_connect(G_OBJECT(tree), "row-activated",G_CALLBACK(pl3_current_playlist_browser_row_activated), NULL); 
-//	g_signal_connect(G_OBJECT(tree), "button-press-event", G_CALLBACK(pl3_current_playlist_browser_button_press_event), NULL);
 	g_signal_connect(G_OBJECT(tree), "button-release-event", G_CALLBACK(pl3_current_playlist_browser_button_release_event), NULL);
 	g_signal_connect(G_OBJECT(tree), "key-press-event", G_CALLBACK(pl3_current_playlist_browser_key_release_event), entry);
 
@@ -427,7 +377,7 @@ static void pl3_current_playlist_browser_init(void)
 	g_object_ref(G_OBJECT(pl3_cp_vbox));
 }
 
-static void pl3_current_playlist_browser_select_current_song()
+static void pl3_current_playlist_browser_select_current_song(void)
 {
 	if(pl3_cp_tree == NULL|| !GTK_WIDGET_REALIZED(pl3_cp_tree)) return;
 	/* scroll to the playing song */
@@ -442,7 +392,7 @@ static void pl3_current_playlist_browser_select_current_song()
 	}      
 }
 
-static void pl3_current_playlist_browser_scroll_to_current_song()
+static void pl3_current_playlist_browser_scroll_to_current_song(void)
 {
 	if(pl3_cp_tree == NULL || !GTK_WIDGET_REALIZED(pl3_cp_tree)) return;
 	/* scroll to the playing song */
@@ -493,7 +443,7 @@ static void pl3_current_playlist_browser_add(GtkWidget *cat_tree)
 /* delete all selected songs,
  * if no songs select ask the user if he want's to clear the list 
  */
-static void pl3_current_playlist_browser_delete_selected_songs ()
+static void pl3_current_playlist_browser_delete_selected_songs (void)
 {
 	/* grab the selection from the tree */
 	GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(pl3_cp_tree));
@@ -528,6 +478,9 @@ static void pl3_current_playlist_browser_delete_selected_songs ()
 
 		/* close the list, so it will be executed */
 		mpd_playlist_queue_commit(connection);
+		/* unselect all if multiple songs were selected */
+		if(g_list_length(list) > 1)
+			gtk_tree_selection_unselect_all(selection);
 		/* free list */
 		g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
 		g_list_free (list);
@@ -537,8 +490,7 @@ static void pl3_current_playlist_browser_delete_selected_songs ()
 		/* create a warning message dialog */
 		GtkWidget *dialog =
 			gtk_message_dialog_new (GTK_WINDOW
-					(glade_xml_get_widget
-					 (pl3_xml, "pl3_win")),
+					(playlist3_get_window()),
 					GTK_DIALOG_MODAL,
 					GTK_MESSAGE_WARNING,
 					GTK_BUTTONS_NONE,
@@ -554,16 +506,17 @@ static void pl3_current_playlist_browser_delete_selected_songs ()
 			case GTK_RESPONSE_OK:
 				/* check if where still connected */
 				mpd_playlist_clear(connection);
+            default:
+                break;
 		}
 		gtk_widget_destroy (GTK_WIDGET (dialog));
 	}
 	/* update everything if where still connected */
-	gtk_tree_selection_unselect_all(selection);
 
 	mpd_status_queue_update(connection);
 }
 
-static void pl3_current_playlist_browser_crop_selected_songs()
+static void pl3_current_playlist_browser_crop_selected_songs(void)
 {
 	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(pl3_cp_tree));
 	/* grab the selection from the tree */
@@ -656,98 +609,8 @@ static void pl3_current_playlist_editor_add_to_playlist(GtkWidget *menu)
 
 	playlist_editor_fill_list();
 }
-static void pl3_current_playlist_browser_edit_columns(void)
-{
-  gmpc_mpddata_treeview_edit_columns(GMPC_MPDDATA_TREEVIEW(pl3_cp_tree));
-}
 
-/**
- * Cut, Copy Paste support
- */
-static void pl3_current_playlist_browser_cut_songs()
-{
-    /* grab the selection from the tree */
-    GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(pl3_cp_tree));
-
-    /** Clear the cut queue */
-    g_queue_foreach(cut_queue, (GFunc)g_free, NULL);
-    g_queue_clear(cut_queue);
-    /* check if where connected */
-    /* see if there is a row selected */
-    if (gtk_tree_selection_count_selected_rows (selection) > 0)
-    {
-        GList *list = NULL, *llist = NULL;
-        GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(pl3_cp_tree));
-        /* start a command list */
-        /* grab the selected songs */
-        list = gtk_tree_selection_get_selected_rows (selection, &model);
-        /* grab the last song that is selected */
-        llist = g_list_last (list);
-        /* remove every selected song one by one */
-        do{
-            GtkTreeIter iter;
-            int value;
-            gtk_tree_model_get_iter (model, &iter,(GtkTreePath *) llist->data);
-            /* Trick that avoids roundtrip to mpd */
-            {
-                char *path = NULL;
-                /* this one allready has the pos. */
-                gtk_tree_model_get (model, &iter, MPDDATA_MODEL_COL_SONG_POS, &value,MPDDATA_MODEL_COL_PATH, &path, -1);			
-                g_queue_push_head(cut_queue, path);
-                value--;
-            } 
-            mpd_playlist_queue_delete_pos(connection, value);			
-        } while ((llist = g_list_previous (llist)));
-        mpd_playlist_queue_commit(connection);
-        /* free list */
-        g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
-        g_list_free (list);
-    }
-    /* update everything if where still connected */
-    gtk_tree_selection_unselect_all(selection);
-}
-static void pl3_current_playlist_browser_copy_songs()
-{
-    /* grab the selection from the tree */
-    GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(pl3_cp_tree));
-
-    /** Clear the cut queue */
-    g_queue_foreach(cut_queue, (GFunc)g_free, NULL);
-    g_queue_clear(cut_queue);
-    /* check if where connected */
-    /* see if there is a row selected */
-    if (gtk_tree_selection_count_selected_rows (selection) > 0)
-    {
-        GList *list = NULL, *llist = NULL;
-        GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(pl3_cp_tree));
-        /* start a command list */
-        /* grab the selected songs */
-        list = gtk_tree_selection_get_selected_rows (selection, &model);
-        /* grab the last song that is selected */
-        llist = g_list_last (list);
-        /* remove every selected song one by one */
-        do{
-            GtkTreeIter iter;
-            int value;
-            gtk_tree_model_get_iter (model, &iter,(GtkTreePath *) llist->data);
-            /* Trick that avoids roundtrip to mpd */
-            {
-                char *path = NULL;
-                /* this one allready has the pos. */
-                gtk_tree_model_get (model, &iter, MPDDATA_MODEL_COL_SONG_POS, &value,MPDDATA_MODEL_COL_PATH, &path, -1);			
-                g_queue_push_head(cut_queue, path);
-                value--;
-            } 
-        } while ((llist = g_list_previous (llist)));
-        /* free list */
-        g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
-        g_list_free (list);
-    }
-    /* update everything if where still connected */
-    gtk_tree_selection_unselect_all(selection);
-}
-
-static void pl3_current_playlist_browser_paste_after_songs()
+static void pl3_current_playlist_browser_paste_after_songs(GtkTreeView *tree, GList *paste_list)
 {
     /* grab the selection from the tree */
     GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(pl3_cp_tree));
@@ -763,7 +626,7 @@ static void pl3_current_playlist_browser_paste_after_songs()
         /* grab the selected songs */
         list = gtk_tree_selection_get_selected_rows (selection, &model);
         /* grab the last song that is selected */
-        llist = g_list_last (list);
+        llist = g_list_last(list);
         /* remove every selected song one by one */
         if(llist){
             GtkTreeIter iter;
@@ -773,18 +636,21 @@ static void pl3_current_playlist_browser_paste_after_songs()
                 int id;
                 char *path = NULL;
                 int length = mpd_playlist_get_playlist_length(connection);
+                GList *liter = g_list_first(paste_list);
                 gtk_tree_model_get (model, &iter, MPDDATA_MODEL_COL_SONG_POS, &id, -1);			
-                while((path = g_queue_pop_tail(cut_queue)))
+                while(liter)
                 {
-                    int song_id = mpd_playlist_add_get_id(connection, path);
-                    if(song_id == -1 && seen)
+                    int song_id;
+                    path = liter->data;
+                    song_id = mpd_playlist_add_get_id(connection, path);
+                    if(song_id == -1 && !seen)
                     {
                         playlist3_show_error_message(_("Your mpd has a broken 'addid', pasting will fail."), ERROR_WARNING);      
                         seen = 1;
                     }
                     mpd_playlist_move_pos(connection, length, id);
-                    g_free(path);
                     length++;
+                    liter = g_list_next(liter);
                 }
             }
         }
@@ -792,22 +658,23 @@ static void pl3_current_playlist_browser_paste_after_songs()
         g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
         g_list_free (list);
     }else{
-            char *path = NULL;
-            while((path = g_queue_pop_head(cut_queue)))
+        GList *liter = g_list_first(paste_list);;
+        while(liter)
+        {
+            char *path = liter->data;
+            int song_id = mpd_playlist_add_get_id(connection, path);
+            if(song_id == -1 && !seen)
             {
-                int song_id = mpd_playlist_add_get_id(connection, path);
-                if(song_id == -1 && seen)
-                {
-                    playlist3_show_error_message(_("Your mpd has a broken 'addid', pasting will fail."), ERROR_WARNING);      
-                    seen = 1;
-                }
-                g_free(path);
+                playlist3_show_error_message(_("Your mpd has a broken 'addid', pasting will fail."), ERROR_WARNING);      
+                seen = 1;
             }
-
+            liter = g_list_next(liter);
         }
 
+    }
+    gtk_tree_selection_unselect_all(selection);
 }
-static void pl3_current_playlist_browser_paste_before_songs()
+static void pl3_current_playlist_browser_paste_before_songs(GtkTreeView *tree, GList *paste_list)
 {
     /* grab the selection from the tree */
     GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(pl3_cp_tree));
@@ -823,7 +690,7 @@ static void pl3_current_playlist_browser_paste_before_songs()
         /* grab the selected songs */
         list = gtk_tree_selection_get_selected_rows (selection, &model);
         /* grab the last song that is selected */
-        llist = g_list_last (list);
+        llist = g_list_first(list);
         /* remove every selected song one by one */
         if(llist){
             GtkTreeIter iter;
@@ -831,23 +698,23 @@ static void pl3_current_playlist_browser_paste_before_songs()
             /* Trick that avoids roundtrip to mpd */
             {
                 int id;
-                char *path = NULL;
                 int length = mpd_playlist_get_playlist_length(connection);
+                GList *liter = g_list_first(paste_list);
                 gtk_tree_model_get (model, &iter, MPDDATA_MODEL_COL_SONG_POS, &id, -1);			
-                while((path = g_queue_pop_head(cut_queue)))
+                while(liter)
                 {
+                    char *path = liter->data;
                     int song_id = mpd_playlist_add_get_id(connection, path);
-                    if(song_id == -1 && seen)
+                    if(song_id == -1 && !seen)
                     {
                         playlist3_show_error_message(_("Your mpd has a broken 'addid', pasting will fail."), ERROR_WARNING);      
                         seen = 1;
                     }
                     mpd_playlist_move_pos(connection, length, id-1);
-                    g_free(path);
                     /* The song is now one lower */
-                    id++;
                     /* length one longer */
                     length++;
+                    liter = g_list_next(liter);
                 }
             }
         }
@@ -855,20 +722,22 @@ static void pl3_current_playlist_browser_paste_before_songs()
         g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
         g_list_free (list);
     }else{
-            char *path = NULL;
-            while((path = g_queue_pop_head(cut_queue)))
+            GList *liter = g_list_first(paste_list);
+            while(liter)
             {
+                char *path = liter->data;
                 int song_id = mpd_playlist_add_get_id(connection, path);
-                if(song_id == -1 && seen)
+                if(song_id == -1 && !seen)
                 {
                     playlist3_show_error_message(_("Your mpd has a broken 'addid', pasting will fail."), ERROR_WARNING);      
                     seen = 1;
                 }
-                g_free(path);
+                liter = g_list_next(liter);
             }
 
         }
 
+    gtk_tree_selection_unselect_all(selection);
 }
 
 
@@ -880,6 +749,7 @@ static int pl3_current_playlist_browser_button_release_event(GtkTreeView *tree,
 		GtkWidget *item;
 		GtkWidget *menu = gtk_menu_new();	
 
+		int rows = gtk_tree_selection_count_selected_rows(gtk_tree_view_get_selection(tree));
 		/* add the delete widget */
 		item = gtk_image_menu_item_new_from_stock(GTK_STOCK_REMOVE,NULL);
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
@@ -891,33 +761,6 @@ static int pl3_current_playlist_browser_button_release_event(GtkTreeView *tree,
 		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_current_playlist_browser_crop_selected_songs), NULL);		
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 
-
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu),gtk_separator_menu_item_new());
-
-        /* Cut/Paste */
-        item = gtk_image_menu_item_new_from_stock(GTK_STOCK_CUT, NULL);
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-        g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_current_playlist_browser_cut_songs), NULL);
-        item = gtk_image_menu_item_new_from_stock(GTK_STOCK_COPY, NULL);
-        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-        g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_current_playlist_browser_copy_songs), NULL);
-
-
-        if(g_queue_get_length(cut_queue) > 0)
-        {
-            item = gtk_image_menu_item_new_with_label(_("Paste before"));
-            gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
-                    gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU));
-            gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-            g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_current_playlist_browser_paste_before_songs), NULL);
-
-            item = gtk_image_menu_item_new_with_label(_("Paste after"));
-            gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
-                    gtk_image_new_from_stock(GTK_STOCK_PASTE, GTK_ICON_SIZE_MENU));
-            gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-            g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_current_playlist_browser_paste_after_songs), NULL);
-        }
-
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu),gtk_separator_menu_item_new());
 		/* add the clear widget */
 		item = gtk_image_menu_item_new_from_stock(GTK_STOCK_CLEAR,NULL);
@@ -934,45 +777,32 @@ static int pl3_current_playlist_browser_button_release_event(GtkTreeView *tree,
 
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu),gtk_separator_menu_item_new());
 
-		item = gtk_image_menu_item_new_from_stock(GTK_STOCK_DIALOG_INFO,NULL);
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_current_playlist_browser_show_info), NULL);		
-
-		/* add the shuffle widget */
-
-
-		item = gtk_image_menu_item_new_with_label(_("Edit Columns"));
-		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
-				gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-		g_signal_connect(G_OBJECT(item), "activate",
-				G_CALLBACK(pl3_current_playlist_browser_edit_columns), NULL);
-
-		/* */
-		if(gtk_tree_selection_count_selected_rows(gtk_tree_view_get_selection(tree)) == 1)	
-		{
-			mpd_Song *song;
-			GtkTreeIter iter;
+        if(rows == 1) {
+            mpd_Song *song;
 			GtkTreePath *path;
 			GtkTreeModel *model = gtk_tree_view_get_model(tree);
+            GtkTreeIter iter;
 			GList *list = gtk_tree_selection_get_selected_rows(gtk_tree_view_get_selection(GTK_TREE_VIEW(tree)), &model);
 			path = list->data;
-			/* free result */
-			g_list_free(list);
-			if(path && gtk_tree_model_get_iter(model, &iter, path)) {
+            if(path && gtk_tree_model_get_iter(model, &iter, path)) {
 				gtk_tree_model_get(model, &iter, MPDDATA_MODEL_COL_MPDSONG, &song, -1);
-				if(song)
-				{
-					submenu_for_song(menu, song);
-				}
-				if(path)                                                                
-					gtk_tree_path_free(path);                                               
-			}
-		}
+				if(song) 
+                {
+                    item = gtk_image_menu_item_new_from_stock(GTK_STOCK_DIALOG_INFO,NULL);
+                    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+                    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_current_playlist_browser_show_info), NULL);		
+
+                    /* Add song sebmenu */
+                    submenu_for_song(menu, song);
+                }
+            }
+            g_list_foreach(list,(GFunc)gtk_tree_path_free, NULL);
+            g_list_free(list);
+        }
+
         
 		playlist_editor_right_mouse(menu,pl3_current_playlist_editor_add_to_playlist);
         gmpc_mpddata_treeview_right_mouse_intergration(GMPC_MPDDATA_TREEVIEW(tree), GTK_MENU(menu));
-
 		gtk_widget_show_all(menu);
 		gtk_menu_popup(GTK_MENU(menu), NULL, NULL,NULL, NULL,0, event->time);	
 		return TRUE;
@@ -992,7 +822,9 @@ static void pl3_current_playlist_browser_row_activated(GtkTreeView *tree, GtkTre
     if(!search_keep_open && model == mod_fill)
     {
         gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_cp_tree), playlist);
+
         gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(mod_fill), NULL);
+        quick_search = 0;
         gtk_widget_hide(filter_entry);
 
         pl3_current_playlist_browser_select_current_song();
@@ -1000,7 +832,7 @@ static void pl3_current_playlist_browser_row_activated(GtkTreeView *tree, GtkTre
 
 }
 
-static void pl3_current_playlist_browser_show_info()
+static void pl3_current_playlist_browser_show_info(void)
 {
 
     GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(pl3_cp_tree));
@@ -1067,7 +899,7 @@ static int pl3_current_playlist_browser_cat_menu_popup(GtkWidget *menu, int type
 
     item = gtk_image_menu_item_new_with_label(_("Add URL"));
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
-		    gtk_image_new_from_icon_name("gmpc-add-url", GTK_ICON_SIZE_MENU));
+		    gtk_image_new_from_icon_name("add-url", GTK_ICON_SIZE_MENU));
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
     g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(url_start), NULL);
 
@@ -1092,26 +924,6 @@ static int  pl3_current_playlist_browser_key_release_event(GtkTreeView *tree, Gd
         pl3_current_playlist_browser_select_current_song();
         return TRUE;			
     }
-    else if (event->keyval == GDK_c && event->state&GDK_CONTROL_MASK)
-    {
-        pl3_current_playlist_browser_copy_songs();
-        return TRUE;
-    }
-    else if (event->keyval == GDK_x && event->state&GDK_CONTROL_MASK)
-    {
-        pl3_current_playlist_browser_cut_songs();
-        return TRUE;
-    }
-    else if (event->keyval == GDK_v && event->state&GDK_CONTROL_MASK)
-    {
-        pl3_current_playlist_browser_paste_after_songs();
-        return TRUE;
-    }
-    else if (event->keyval == GDK_b && event->state&GDK_CONTROL_MASK)
-    {
-        pl3_current_playlist_browser_paste_before_songs();
-        return TRUE;
-    }
     else if (event->keyval == GDK_f && event->state&GDK_CONTROL_MASK)
     {
         mod_fill_entry_changed(entry, NULL);
@@ -1123,26 +935,26 @@ static int  pl3_current_playlist_browser_key_release_event(GtkTreeView *tree, Gd
             ((event->keyval >= GDK_space && event->keyval <= GDK_z)))*/
     {
         char data[10];
-       	gunichar uc = gdk_keyval_to_unicode(event->keyval);
-	if(uc)
-	{
-		memset(data,'\0',10);
-		g_unichar_to_utf8 (uc, data);
-		gtk_widget_grab_focus(entry);      
-		gtk_entry_set_text(GTK_ENTRY(entry),data);
-		gtk_editable_set_position(GTK_EDITABLE(entry),1);
+        guint32 uc = gdk_keyval_to_unicode(event->keyval);
+        if(uc)
+        {
+            memset(data,'\0',10);
+            g_unichar_to_utf8 (uc, data);
+            gtk_widget_grab_focus(entry);      
+            gtk_entry_set_text(GTK_ENTRY(entry),data);
+            gtk_editable_set_position(GTK_EDITABLE(entry),1);
 
-		return TRUE;
-	}
+            return TRUE;
+        }
     }
     return pl3_window_key_press_event(GTK_WIDGET(tree),event);
 }
 
 /* create a dialog that allows the user to save the current playlist */
-static void pl3_current_playlist_save_playlist ()
+static void pl3_current_playlist_save_playlist (void)
 {
     gchar *str;
-    GladeXML *xml = NULL;
+    GtkBuilder *xml = NULL;
     int run = TRUE;
     /* check if the connection is up */
     if (!mpd_check_connected(connection))
@@ -1150,21 +962,22 @@ static void pl3_current_playlist_save_playlist ()
         return;
     }
     /* create the interface */
-    str = gmpc_get_full_glade_path("playlist3.glade");
-    xml = glade_xml_new (str, "save_pl", NULL);
+    str = gmpc_get_full_glade_path("playlist-save-dialog.ui");
+    xml = gtk_builder_new();
+    gtk_builder_add_from_file(xml, str, NULL);
     q_free(str);
 
     /* run the interface */
     do
     {
-        switch (gtk_dialog_run (GTK_DIALOG (glade_xml_get_widget (xml, "save_pl"))))
+        switch (gtk_dialog_run (GTK_DIALOG ((GtkWidget *) gtk_builder_get_object (xml, "save_pl"))))
         {
             case GTK_RESPONSE_OK:
                 run = FALSE;
                 /* if the users agrees do the following: */
                 /* get the song-name */
                 str = (gchar *)	gtk_entry_get_text (GTK_ENTRY
-                        (glade_xml_get_widget (xml, "pl-entry")));
+                        ((GtkWidget *) gtk_builder_get_object (xml, "pl-entry")));
                 /* check if the user entered a name, we can't do withouth */
                 /* TODO: disable ok button when nothing is entered */
                 /* also check if there is a connection */
@@ -1174,11 +987,11 @@ static void pl3_current_playlist_save_playlist ()
                     if(retv == MPD_DATABASE_PLAYLIST_EXIST )
                     {
                         gchar *errormsg = g_markup_printf_escaped(_("<i>Playlist <b>\"%s\"</b> already exists\nOverwrite?</i>"), str);
-                        gtk_label_set_markup(GTK_LABEL(glade_xml_get_widget(xml, "label_error")), errormsg);
-                        gtk_widget_show(glade_xml_get_widget(xml, "hbox5"));
+                        gtk_label_set_markup(GTK_LABEL((GtkWidget *) gtk_builder_get_object(xml, "label_error")), errormsg);
+                        gtk_widget_show((GtkWidget *)gtk_builder_get_object(xml, "hbox5"));
                         /* ask to replace */
-                        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(xml, "pl-entry")), FALSE);
-                        switch (gtk_dialog_run (GTK_DIALOG (glade_xml_get_widget (xml, "save_pl"))))
+                        gtk_widget_set_sensitive(GTK_WIDGET((GtkWidget *) gtk_builder_get_object(xml, "pl-entry")), FALSE);
+                        switch (gtk_dialog_run (GTK_DIALOG ((GtkWidget *) gtk_builder_get_object (xml, "save_pl"))))
                         {
                             case GTK_RESPONSE_OK:
                                 run = FALSE;
@@ -1190,8 +1003,8 @@ static void pl3_current_playlist_save_playlist ()
                                 run = TRUE;
                         }
                         /* return to stare */
-                        gtk_widget_set_sensitive(GTK_WIDGET(glade_xml_get_widget(xml, "pl-entry")), TRUE);
-                        gtk_widget_hide(glade_xml_get_widget(xml, "hbox5"));
+                        gtk_widget_set_sensitive(GTK_WIDGET((GtkWidget *) gtk_builder_get_object(xml, "pl-entry")), TRUE);
+                        gtk_widget_hide((GtkWidget *) gtk_builder_get_object(xml, "hbox5"));
 
                         q_free(errormsg);
                     }
@@ -1210,18 +1023,18 @@ static void pl3_current_playlist_save_playlist ()
         }
     }while(run);
     /* destroy the window */
-    gtk_widget_destroy (glade_xml_get_widget (xml, "save_pl"));
+    gtk_widget_destroy ((GtkWidget *) gtk_builder_get_object (xml, "save_pl"));
 
     /* unref the gui description */
     g_object_unref (xml);
 }
 
-static void pl3_current_playlist_browser_clear_playlist()
+static void pl3_current_playlist_browser_clear_playlist(void)
 {
     mpd_playlist_clear(connection);
 }
 
-static void pl3_current_playlist_browser_shuffle_playlist()
+static void pl3_current_playlist_browser_shuffle_playlist(void)
 {
     mpd_playlist_shuffle(connection);
 }
@@ -1232,7 +1045,9 @@ static void pl3_current_playlist_status_changed(MpdObj *mi, ChangedStatusType wh
         return;
     if(what&MPD_CST_PLAYLIST)
     {
-        mod_fill_do_entry_changed(filter_entry, NULL);
+
+        if(quick_search)
+            mod_fill_do_entry_changed(filter_entry, NULL);
         if(pl3_curb_tree_ref) {
             GtkTreeIter iter;
             GtkTreePath *path;
@@ -1253,7 +1068,7 @@ static void pl3_current_playlist_status_changed(MpdObj *mi, ChangedStatusType wh
     }
 }
 
-static void pl3_current_playlist_browser_activate()
+static void pl3_current_playlist_browser_activate(void)
 {
     GtkTreeSelection *selec = gtk_tree_view_get_selection((GtkTreeView *)
             glade_xml_get_widget (pl3_xml, "cat_tree"));
@@ -1313,6 +1128,3 @@ static void pl3_current_playlist_save_myself(void)
         }
     }
 }
-
-
-
