@@ -1,42 +1,35 @@
-/*
- *Copyright (C) 2004-2008 Qball Cow <qball@sarine.nl>
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
+/* Gnome Music Player Client (GMPC)
+ * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Project homepage: http://gmpc.wikia.com/
+ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- */
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
 
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdlib.h>
 #include <string.h>
-#include <glade/glade.h>
-
-#include "plugin.h"
-
 #include "main.h"
 #include "misc.h"
 #include "playlist3.h"
 #include "playlist3-file-browser.h"
-#include "config1.h"
 #include "TreeSearchWidget.h"
 #include "gmpc-mpddata-model.h"
 #include "gmpc-mpddata-treeview.h"
 #include "playlist3-playlist-editor.h"
 
-static void pl3_file_browser_edit_columns(void);
-
 static void pl3_file_browser_destroy(void);
 static void pl3_file_browser_add(GtkWidget *cat_tree);
 static void pl3_file_browser_unselected(GtkWidget *container);
@@ -104,7 +97,7 @@ gmpcPlBrowserPlugin file_browser_gbp = {
 gmpcPlugin file_browser_plug = {
 	.name = 						N_("File Browser"),
 	.version = 						{1,1,1},
-	.plugin_type = 					GMPC_PLUGIN_PL_BROWSER,
+	.plugin_type = 					GMPC_PLUGIN_PL_BROWSER|GMPC_INTERNALL,
   	.destroy = 						pl3_file_browser_destroy,
 	.browser = 						&file_browser_gbp,
 	.mpd_status_changed = 			pl3_file_browser_status_changed,
@@ -115,9 +108,6 @@ gmpcPlugin file_browser_plug = {
 };
 
 
-extern GladeXML *pl3_xml;
-
-
 /* internal */
 GtkWidget *pl3_fb_tree = NULL;
 GtkWidget *pl3_fb_vbox = NULL;
@@ -125,8 +115,9 @@ GtkWidget *pl3_fb_tree_search = NULL;
 GmpcMpdDataModel *pl3_fb_store2 = NULL;
 static GtkTreeStore *pl3_fb_dir_store = NULL;
 static GtkWidget *pl3_fb_dir_tree = NULL;
+static GtkWidget *pl3_fb_warning_box = NULL;
 
-static void pl3_file_browser_search_activate()
+static void pl3_file_browser_search_activate(void)
 {
 	GtkTreeModel *model = GTK_TREE_MODEL(pl3_fb_store2);
 	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(pl3_fb_tree));
@@ -156,7 +147,7 @@ static void playtime_changed(GmpcMpdDataModel *model, gulong playtime)
     }
 }
 
-static void pl3_file_browser_init()
+static void pl3_file_browser_init(void)
 {
     GtkCellRenderer *renderer;
     GtkTreeViewColumn *column;
@@ -231,6 +222,18 @@ static void pl3_file_browser_init()
 	gtk_box_pack_end(GTK_BOX(vbox), pl3_fb_tree_search, FALSE, TRUE,0);
 	g_signal_connect(G_OBJECT(pl3_fb_tree_search),"result-activate", G_CALLBACK(pl3_file_browser_search_activate), NULL);
 
+
+        /* Warning box for when there is no music */
+        pl3_fb_warning_box = gtk_label_new("");
+        gtk_label_set_markup(GTK_LABEL(pl3_fb_warning_box), 
+	_("It seems you have no music in your database.\n"
+	  "To add music, copy the music to your <i>music_directory</i> as specified in your mpd config file.\n"
+          "Then update the database. (Server->Update Database)"));
+        gtk_misc_set_alignment(GTK_MISC(pl3_fb_warning_box), 0.0,0.0);
+        gtk_misc_set_padding(GTK_MISC(pl3_fb_warning_box), 12,12);
+	gtk_widget_set_no_show_all(pl3_fb_warning_box, TRUE);
+	gtk_box_pack_end(GTK_BOX(vbox), pl3_fb_warning_box, FALSE, TRUE,0);
+
     gtk_paned_add2(GTK_PANED(pl3_fb_vbox), vbox);
 	/* set initial state */
 	debug_printf(DEBUG_INFO,"initialized current playlist treeview\n");
@@ -239,7 +242,7 @@ static void pl3_file_browser_init()
 	g_object_ref(G_OBJECT(pl3_fb_vbox));
 }
 
-static void pl3_file_browser_add_folder()
+static void pl3_file_browser_add_folder(void)
 {
 	GtkTreeSelection *selec = gtk_tree_view_get_selection(GTK_TREE_VIEW(pl3_fb_dir_tree));
 	GtkTreeModel *model = GTK_TREE_MODEL(pl3_fb_dir_store);
@@ -272,7 +275,7 @@ static void pl3_file_browser_add_folder()
 }
 
 
-static void pl3_file_browser_update_folder()
+static void pl3_file_browser_update_folder(void)
 {
 	GtkTreeSelection *selec = gtk_tree_view_get_selection((GtkTreeView *)pl3_fb_dir_tree);
 	GtkTreeModel *model = GTK_TREE_MODEL(pl3_fb_dir_store);
@@ -295,7 +298,7 @@ static void pl3_file_browser_update_folder()
 
 }
 
-static void pl3_file_browser_update_folder_left_pane()
+static void pl3_file_browser_update_folder_left_pane(void)
 {
 	GtkTreeModel *model = GTK_TREE_MODEL(pl3_fb_store2);
 	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(pl3_fb_tree));
@@ -325,7 +328,7 @@ static void pl3_file_browser_update_folder_left_pane()
 
 
 
-static void pl3_file_browser_replace_folder()
+static void pl3_file_browser_replace_folder(void)
 {
 	mpd_playlist_clear(connection);
     if(mpd_check_connected(connection))
@@ -378,9 +381,7 @@ static void pl3_file_browser_add(GtkWidget *cat_tree)
             PL3_FB_PATH,"/",
             PL3_FB_OPEN,FALSE,
 			-1);
-    gtk_tree_store_append(pl3_fb_dir_store, &child, &iter);
-
-
+    gtk_tree_store_append(pl3_fb_dir_store, &child, &iter);	
 }
          
 static void pl3_file_browser_reupdate_folder(GtkTreeIter *iter)
@@ -411,41 +412,44 @@ static void pl3_file_browser_reupdate_folder(GtkTreeIter *iter)
 				{
 					int compare =0;
 					/* get the next directory */
-					while(data->type != MPD_DATA_TYPE_DIRECTORY) data = mpd_data_get_next(data);
-					compare = strcmp(data->directory, test_path);
-					if(compare < 0)
-					{
-						gchar *basename = g_path_get_basename (data->directory);
-						gtk_tree_store_insert_before(pl3_fb_dir_store, &child2,iter,&child);
-						gtk_tree_store_set (pl3_fb_dir_store, &child2,
-                                PL3_FB_ICON, "gtk-open",
-								PL3_FB_NAME, basename,
-								PL3_FB_PATH, data->directory,
-								PL3_FB_OPEN, FALSE,
-								-1);
+					while(data && data->type != MPD_DATA_TYPE_DIRECTORY) data = mpd_data_get_next(data);
+                    if(data)
+                    {
+                        compare = strcmp(data->directory, test_path);
+                        if(compare < 0)
+                        {
+                            gchar *basename = g_path_get_basename (data->directory);
+                            gtk_tree_store_insert_before(pl3_fb_dir_store, &child2,iter,&child);
+                            gtk_tree_store_set (pl3_fb_dir_store, &child2,
+                                    PL3_FB_ICON, "gtk-open",
+                                    PL3_FB_NAME, basename,
+                                    PL3_FB_PATH, data->directory,
+                                    PL3_FB_OPEN, FALSE,
+                                    -1);
 
-						gtk_tree_store_append(pl3_fb_dir_store, &child3, &child2);
-						q_free(basename);
+                            gtk_tree_store_append(pl3_fb_dir_store, &child3, &child2);
+                            q_free(basename);
 
-						/* if the new dir is smaller the temp, we add it. */
-						data = mpd_data_get_next(data);
-						has_next = TRUE;
-					}
-                    else if(compare > 0)
-					{	
-						/* if it's bigger, we delete the row */
-						has_next = gtk_tree_store_remove(pl3_fb_dir_store, &child);
+                            /* if the new dir is smaller the temp, we add it. */
+                            data = mpd_data_get_next(data);
+                            has_next = TRUE;
+                        }
+                        else if(compare > 0)
+                        {	
+                            /* if it's bigger, we delete the row */
+                            has_next = gtk_tree_store_remove(pl3_fb_dir_store, &child);
 
-					}else{
-					   /* if equal we process children if available */
-						if(temp)
-						{
-							pl3_file_browser_reupdate_folder(&child);
-						}
-						/* move to next entry in both */
-						has_next = gtk_tree_model_iter_next(model, &child);					
-						data = mpd_data_get_next(data);
-					}
+                        }else{
+                            /* if equal we process children if available */
+                            if(temp)
+                            {
+                                pl3_file_browser_reupdate_folder(&child);
+                            }
+                            /* move to next entry in both */
+                            has_next = gtk_tree_model_iter_next(model, &child);					
+                            data = mpd_data_get_next(data);
+                        }
+                    }
 				}
 				g_free(test_path);
 			}while(has_next);
@@ -471,7 +475,7 @@ static void pl3_file_browser_reupdate_folder(GtkTreeIter *iter)
 		}
 	}
 }
-static void pl3_file_browser_reupdate()
+static void pl3_file_browser_reupdate(void)
 {
 
 	if(pl3_fb_vbox)
@@ -479,9 +483,23 @@ static void pl3_file_browser_reupdate()
 		GtkTreeIter iter;
 
 		GtkTreeModel *model = GTK_TREE_MODEL(pl3_fb_dir_store); 
+		
+		if(mpd_stats_get_total_songs(connection) == 0)
+		{
+                    gtk_widget_show(pl3_fb_warning_box);
+		}else{
+                    gtk_widget_hide(pl3_fb_warning_box);
+		}
+
 
 		if(gtk_tree_model_get_iter_first(model, &iter))
 		{
+            GtkTreeIter child;
+            if(!gtk_tree_model_iter_children(model, &child, &iter))
+            {
+                gtk_tree_store_set(GTK_TREE_STORE(model), &iter, PL3_FB_OPEN, FALSE, -1);
+                gtk_tree_store_append(pl3_fb_dir_store, &child, &iter);
+            }
 			pl3_file_browser_reupdate_folder(&iter);
             pl3_file_browser_view_folder(gtk_tree_view_get_selection(GTK_TREE_VIEW(pl3_fb_dir_tree)), NULL);
         }
@@ -639,9 +657,9 @@ static int pl3_file_browser_cat_popup(GtkWidget *wid, GdkEventButton *event, gpo
 
 static gboolean pl3_file_browser_cat_key_press(GtkWidget *tree, GdkEventKey *event,gpointer data)
 {
-    if(event->state&GDK_CONTROL_MASK && event->keyval == GDK_Insert) {
+    if(event->state&GDK_CONTROL_MASK && (event->keyval == GDK_Insert || event->keyval == GDK_KP_Insert)) {
         pl3_file_browser_replace_folder();
-    } else if(event->keyval == GDK_Insert) {
+    } else if(event->keyval == GDK_Insert || event->keyval == GDK_KP_Insert) {
         pl3_file_browser_add_folder();
     }
     return FALSE;
@@ -649,12 +667,12 @@ static gboolean pl3_file_browser_cat_key_press(GtkWidget *tree, GdkEventKey *eve
 
 static int pl3_file_browser_playlist_key_press(GtkWidget *tree, GdkEventKey *event)
 {
-    if(event->state&GDK_CONTROL_MASK && event->keyval == GDK_Insert)
+    if(event->state&GDK_CONTROL_MASK && (event->keyval == GDK_Insert || event->keyval == GDK_KP_Insert))
     {
         pl3_file_browser_replace_selected();
     } else if (event->state&GDK_CONTROL_MASK && event->keyval == GDK_f) {
         treesearch_start(TREESEARCH(pl3_fb_tree_search));
-    } else if(event->keyval == GDK_Insert) {
+    } else if(event->keyval == GDK_Insert || event->keyval == GDK_KP_Insert) {
         pl3_file_browser_add_selected();
     } else if(event->keyval == GDK_i && event->state&GDK_MOD1_MASK) {
         pl3_file_browser_show_info();
@@ -689,7 +707,7 @@ static void pl3_file_browser_unselected(GtkWidget *container)
     gtk_container_remove(GTK_CONTAINER(container), pl3_fb_vbox);
 }
 
-static void pl3_file_browser_show_info()
+static void pl3_file_browser_show_info(void)
 {
     GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(pl3_fb_tree));
     GtkTreeSelection *selection =gtk_tree_view_get_selection (GTK_TREE_VIEW(pl3_fb_tree));
@@ -704,22 +722,21 @@ static void pl3_file_browser_show_info()
 
         list = g_list_last (list);
         {
+            mpd_Song *song = NULL;
             GtkTreeIter iter;
-            char *path;
             int type;
             gtk_tree_model_get_iter (model, &iter, (GtkTreePath *) list->data);
-            gtk_tree_model_get (GTK_TREE_MODEL(pl3_fb_store2), &iter,MPDDATA_MODEL_ROW_TYPE,&type, MPDDATA_MODEL_COL_PATH, &path, -1);
+            gtk_tree_model_get (GTK_TREE_MODEL(pl3_fb_store2), &iter,
+                        MPDDATA_MODEL_ROW_TYPE,&type, 
+                        MPDDATA_MODEL_COL_MPDSONG, &song,-1);
             if(type == MPD_DATA_TYPE_SONG)
             {
-                mpd_Song *song = mpd_database_get_fileinfo(connection, path);
                 if(song)
                 {
                     info2_activate();
                     info2_fill_song_view(song);	
-                    mpd_freeSong(song);
                 }
             }
-            q_free(path);
         }
         g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
         g_list_free (list);
@@ -801,10 +818,6 @@ static void pl3_file_browser_row_activated(GtkTreeView *tree, GtkTreePath *tp)
 
     q_free(song_path);
 }
-static void pl3_file_browser_edit_columns(void)
-{
-    gmpc_mpddata_treeview_edit_columns(GMPC_MPDDATA_TREEVIEW(pl3_fb_tree));
-}
 
 static void pl3_file_browser_add_to_playlist(GtkWidget *menu)
 {
@@ -918,20 +931,14 @@ static gboolean pl3_file_browser_button_release_event(GtkWidget *but, GdkEventBu
                 has_item = 1;
             }
 
-///            model = gtk_tree_view_get_model(GTK_TREE_VIEW(pl3_fb_tree));
-  //          list = gtk_tree_selection_get_selected_rows(sel,&model);
-    //        if(list)
-                tree_path = list->data;
-                if(tree_path && gtk_tree_model_get_iter(model, &iter, tree_path)) {
-                    gtk_tree_model_get(model, &iter, MPDDATA_MODEL_COL_MPDSONG, &song, -1);
-                    if(song)
-                    {
-                        submenu_for_song(menu, song);
-                    }
-
+            tree_path = list->data;
+            if(tree_path && gtk_tree_model_get_iter(model, &iter, tree_path)) {
+                gtk_tree_model_get(model, &iter, MPDDATA_MODEL_COL_MPDSONG, &song, -1);
+                if(song)
+                {
+                    submenu_for_song(menu, song);
                 }
-            //    g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
-              //  g_list_free(list);
+            }
             g_list_foreach (list,(GFunc) gtk_tree_path_free, NULL);
             g_list_free (list);
             q_free(path);
@@ -955,13 +962,14 @@ static gboolean pl3_file_browser_button_release_event(GtkWidget *but, GdkEventBu
     gmpc_mpddata_treeview_right_mouse_intergration(GMPC_MPDDATA_TREEVIEW(pl3_fb_tree), GTK_MENU(menu));
     /*	if(has_item)*/
     {
+    /*
         item = gtk_image_menu_item_new_with_label(_("Edit Columns"));
         gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
                 gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
         gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
         g_signal_connect(G_OBJECT(item), "activate",
                 G_CALLBACK(pl3_file_browser_edit_columns), NULL);
-
+*/
         gtk_widget_show_all(menu);
         gtk_menu_popup(GTK_MENU(menu), NULL, NULL,NULL, NULL, 0, event->time);
         return TRUE;
@@ -972,7 +980,7 @@ static gboolean pl3_file_browser_button_release_event(GtkWidget *but, GdkEventBu
         */
     return FALSE;
 }
-static void pl3_file_browser_replace_selected()
+static void pl3_file_browser_replace_selected(void)
 {
     mpd_playlist_clear(connection);
     if(mpd_check_connected(connection))
@@ -981,7 +989,7 @@ static void pl3_file_browser_replace_selected()
         mpd_player_play(connection);
     }
 }
-static void pl3_file_browser_add_selected()
+static void pl3_file_browser_add_selected(void)
 {
     GtkTreeIter iter;
     GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(pl3_fb_tree));
@@ -1143,7 +1151,7 @@ static void pl3_file_browser_delete_playlist(GtkMenuItem *bt)
     q_free(path);
 }
 
-static void pl3_file_browser_disconnect()
+static void pl3_file_browser_disconnect(void)
 {
 
     if(pl3_fb_tree_ref) {
@@ -1206,9 +1214,22 @@ static int pl3_file_browser_add_go_menu(GtkWidget *menu)
 
 static void pl3_file_browser_connection_changed(MpdObj *mi, int connect, gpointer data)
 {
-    if(!connect) {
-        pl3_file_browser_disconnect();
-    }
+	if(connect)
+	{
+		GtkTreePath* path = gtk_tree_path_new_from_string("0");
+		if(mpd_stats_get_total_songs(connection) == 0){
+                    gtk_widget_show(pl3_fb_warning_box);
+		}else{
+                    gtk_widget_hide(pl3_fb_warning_box);
+		}
+
+
+
+		gtk_tree_view_expand_to_path(GTK_TREE_VIEW(pl3_fb_dir_tree), path);
+		gtk_tree_path_free(path);
+	}
+	else
+		pl3_file_browser_disconnect();
 }
 
 static void pl3_file_browser_status_changed(MpdObj *mi,ChangedStatusType what, void *data)
