@@ -1,16 +1,32 @@
+/* Gnome Music Player Client (GMPC)
+ * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Project homepage: http://gmpc.wikia.com/
+ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
+
 #include <gtk/gtk.h>
 #include <glib/gstdio.h>
 #include <gdk/gdkkeysyms.h>
 #include <libmpd/libmpd.h>
 #include <string.h>
-
-#include "plugin.h"
-
+#include "main.h"
 #include "gmpc-mpddata-model.h"
 #include "gmpc-mpddata-treeview.h"
 #include "gmpc-mpddata-model-sort.h"
 #include "browsers/playlist3-playlist-editor.h"
-#include "main.h"
 
 #define DEFAULT_MARKUP_BROWSER 	"[%name%: &[%artist% - ]%title%]|%name%|[%artist% - ]%title%|%shortfile%|"
 
@@ -60,7 +76,7 @@ gmpcPlBrowserPlugin playlist_editor_gbp = {
 gmpcPlugin playlist_editor_plugin = {
   .name 					= "Playlist Editor",
   .version					=  {0,15,0},
-  .plugin_type 				= GMPC_PLUGIN_PL_BROWSER,
+  .plugin_type 				= GMPC_PLUGIN_PL_BROWSER|GMPC_INTERNALL,
   .init						= playlist_editor_init,
   .destroy 					= playlist_editor_destroy, 
   .browser					=  &playlist_editor_gbp,
@@ -157,7 +173,6 @@ static void playlist_editor_browser_playlist_editor_selected(GtkIconView *giv, G
 }
 static void playlist_editor_browser_playlist_editor_changed(GtkWidget *giv, gpointer data)
 {
-
 	gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(playlist_editor_list_store), NULL);		
 	gmpc_mpddata_model_sort_set_playlist(GMPC_MPDDATA_MODEL_SORT(playlist_editor_list_store), NULL);
 	/* iter all the selected items (aka 1) */
@@ -239,22 +254,17 @@ loop:
                     else if (val > 0) {
                         GtkTreeIter niter;
                         GdkPixbuf *pb = NULL; 
-                        pb = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "media-playlist", 64, 0,NULL);
+                        if(g_utf8_collate(data->playlist->path, _("Favorites"))){
+                            pb = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "media-playlist", 32, 0,NULL);
+                        }else{
+                            pb = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "emblem-favorite",32, 0,NULL);
+                        }
                         gtk_list_store_insert_before(playlist_editor_store,&niter, &iter);
                         gtk_list_store_set(playlist_editor_store, &niter,PL_NAME, data->playlist->path, PL_MTIME, data->playlist->mtime, PL_IMAGE, pb, -1);
                         if(pb)
                             g_object_unref(pb);
                     }else{
-                    /*
-                        GtkTreeIter niter;
-                        GdkPixbuf *pb = NULL; 
-                        pb = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "media-playlist", 64, 0,NULL);
-                        gtk_list_store_insert_after(playlist_editor_store,&niter, &iter);
-                        gtk_list_store_set(playlist_editor_store, &niter,PL_NAME, data->playlist->path, PL_MTIME, data->playlist->mtime, PL_IMAGE, pb, -1);
-                        if(pb)
-                            g_object_unref(pb);
-                        iter = niter;
-                        */
+
                         valid = gtk_list_store_remove(playlist_editor_store, &iter);
                         g_free(name);
                         goto loop;
@@ -265,25 +275,17 @@ loop:
                 else
                 {
                     GdkPixbuf *pb = NULL; 
-                    pb = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "media-playlist", 64, 0,NULL);
+
+                    if(g_utf8_collate(data->playlist->path, _("Favorites"))){
+                        pb = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "media-playlist", 32, 0,NULL);
+                    }else{
+                        pb = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "emblem-favorite",32, 0,NULL);
+                    }
                     gtk_list_store_append(playlist_editor_store, &iter);
                     gtk_list_store_set(playlist_editor_store, &iter,PL_NAME, data->playlist->path, PL_MTIME, data->playlist->mtime, PL_IMAGE, pb, -1);
                     if(pb)
                         g_object_unref(pb);
                     
-                        /*
-                    if(!selected)
-                    {
-                        GtkTreePath *path;
-                        path = gtk_tree_model_get_path(GTK_TREE_MODEL(playlist_editor_store), &iter);
-                        if(path)
-                        {
-                            gtk_icon_view_select_path(GTK_ICON_VIEW(playlist_editor_icon_view), path);
-                            gtk_tree_path_free(path);
-                            selected = TRUE;
-                        }
-                    }
-                    */
                 }
                 
 			}
@@ -310,6 +312,8 @@ loop:
         }
 
 	}
+    if(playlist_editor_icon_view)
+    gtk_icon_view_selected_foreach(GTK_ICON_VIEW(playlist_editor_icon_view), playlist_editor_browser_playlist_editor_selected, NULL);
 }
 static void playlist_editor_browser_activate_cursor_item(GtkIconView *giv, gpointer userdata)
 {
@@ -382,6 +386,7 @@ static void playlist_editor_list_delete_songs(GtkButton *button, GtkTreeView *tr
 
 static void playlist_editor_list_add_songs(GtkButton *button, GtkTreeView *tree) 
 {
+    int songs  = 0;
 	GtkTreeSelection *sel= gtk_tree_view_get_selection(tree);
 	GtkTreeModel *model = gtk_tree_view_get_model(tree);
 	GList *data,*list= gtk_tree_selection_get_selected_rows(sel, &model);
@@ -394,13 +399,26 @@ static void playlist_editor_list_add_songs(GtkButton *button, GtkTreeView *tree)
 			gchar *song_path;
 			gtk_tree_model_get(model, &iter,MPDDATA_MODEL_COL_PATH, &song_path, -1); 
 			mpd_playlist_queue_add(connection, song_path);
-
+            songs++;
 			g_free(song_path);
 		}
 	}
 	mpd_playlist_queue_commit(connection);
 	g_list_foreach (list,(GFunc) gtk_tree_path_free, NULL);
 	g_list_free (list);
+    if(songs > 0)
+    {
+        gchar *added = g_strdup_printf("%s %i %s.", _("Added"), songs, ngettext("song", "songs", songs));
+        pl3_push_statusbar_message(added);
+        g_free(added);
+
+    }
+}
+static void playlist_editor_list_replace_songs(GtkButton *button, GtkTreeView *tree)
+{
+    mpd_playlist_clear(connection);
+    playlist_editor_list_add_songs(button, tree);
+    mpd_player_play(connection);
 }
 static void playlist_editor_clear_playlist(GtkWidget *item, gpointer data)
 {
@@ -445,7 +463,7 @@ static void playlist_editor_new_entry_changed(GtkEntry *entry, GtkWidget *button
 static void playlist_editor_new_playlist(GtkWidget *item, gpointer data)
 {
 	int done = 0;
-    GtkWidget *pl3_win = glade_xml_get_widget(pl3_xml, "pl3_win");
+    GtkWidget *pl3_win = playlist3_get_window(); 
     GtkWidget *dialog = gtk_dialog_new_with_buttons(_("New playlist"), NULL,GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 				NULL);
@@ -520,11 +538,6 @@ static void playlist_editor_rename_playlist(GtkWidget *item, gpointer data)
 	gtk_widget_destroy(dialog);
 }
 
-static void playlist_editor_edit_columns(void)
-{
-  gmpc_mpddata_treeview_edit_columns(GMPC_MPDDATA_TREEVIEW(playlist_editor_song_tree));
-}
-
 static gboolean playlist_editor_key_released(GtkTreeView *tree, GdkEventButton *button, gpointer data)
 {
 	if(button->button == 3)
@@ -539,21 +552,22 @@ static gboolean playlist_editor_key_released(GtkTreeView *tree, GdkEventButton *
         item = gtk_image_menu_item_new_from_stock(GTK_STOCK_ADD,NULL);
         gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
         g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(playlist_editor_list_add_songs), tree);
+
+        item = gtk_image_menu_item_new_with_label(_("Replace"));
+        gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
+                gtk_image_new_from_stock(GTK_STOCK_REDO, GTK_ICON_SIZE_MENU));
+        gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+        g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(playlist_editor_list_replace_songs), tree);
       	if(mpd_server_check_version(connection, 0,13,0))
         {
-          item = gtk_image_menu_item_new_from_stock(GTK_STOCK_DELETE,NULL);
+          item = gtk_image_menu_item_new_from_stock(GTK_STOCK_REMOVE,NULL);
           gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
           g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(playlist_editor_list_delete_songs), tree);
         }
       }
-      item = gtk_image_menu_item_new_with_label(_("Edit Columns"));
-      gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
-          gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
-      gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-      g_signal_connect(G_OBJECT(item), "activate",
-          G_CALLBACK(playlist_editor_edit_columns), NULL);
-
 
+     
+      gmpc_mpddata_treeview_right_mouse_intergration(GMPC_MPDDATA_TREEVIEW(tree), GTK_MENU(menu));
 
 
       gtk_widget_show_all(menu);
@@ -562,21 +576,20 @@ static gboolean playlist_editor_key_released(GtkTreeView *tree, GdkEventButton *
   }
   return FALSE;
 }
-static gboolean playlist_editor_key_pressed(GtkWidget *giv, GdkEventButton *event, gpointer data)
+static gboolean playlist_editor_key_pressed(GtkWidget *giv, GdkEventKey *event, gpointer data)
 {
-	GtkTreePath *path = NULL;
-	if(event->button == 3 &&gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(giv), event->x, event->y,&path,NULL,NULL,NULL))
-	{	
-		GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(giv));
-		if(gtk_tree_selection_path_is_selected(sel, path))
-		{
-			gtk_tree_path_free(path);
-			return TRUE;
-		}
-	}
-	if(path) {
-		gtk_tree_path_free(path);
-	}
+    if(event->keyval == GDK_Delete)
+    {
+        playlist_editor_list_delete_songs(NULL, GTK_TREE_VIEW(giv)); 
+    }
+    else if(event->state&GDK_CONTROL_MASK && (event->keyval == GDK_Insert || event->keyval == GDK_KP_Insert)) {
+        playlist_editor_list_replace_songs(NULL, GTK_TREE_VIEW(giv)); 
+    }
+    else if (event->keyval == GDK_Insert || event->keyval == GDK_KP_Insert)
+    {
+        playlist_editor_list_add_songs(NULL, GTK_TREE_VIEW(giv)); 
+    }
+  
 	return FALSE;                                                                                                     
 }
 static gboolean playlist_editor_browser_button_release_event(GtkWidget *giv, GdkEventButton *event, gpointer data)
@@ -656,21 +669,179 @@ static void playtime_changed(GmpcMpdDataModel *model, gulong playtime)
         playlist3_show_playtime(playtime);
     }
 }
+/**
+ * Selected songs are cut, now it has to be deleted
+ */
+static void playlist_editor_cut_songs(GtkTreeView *tree)
+{
+    playlist_editor_list_delete_songs(NULL, tree);
+}
+static void playlist_editor_paste_after_songs(GtkTreeView *tree, GList *paste_list)
+{
+	gchar *pl_path = NULL;
+	GList *it,*list = gtk_icon_view_get_selected_items(GTK_ICON_VIEW(playlist_editor_icon_view));
+	for(it = g_list_first(list);it; it = g_list_next(it))
+	{
+		GtkTreePath *path = it->data;
+		GtkTreeIter iter;
+		if(gtk_tree_model_get_iter(GTK_TREE_MODEL(playlist_editor_store), &iter, path) && !pl_path)
+		{
+			gtk_tree_model_get(GTK_TREE_MODEL(playlist_editor_store), &iter, PL_NAME, &pl_path, -1);
+		}
+	}
+	g_list_foreach(list, (GFunc)gtk_tree_path_free, NULL);
+	g_list_free(list);
 
+	if(pl_path)
+    {
+        /* grab the selection from the tree */
+        GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(tree));
+        /* check if where connected */
+        /* see if there is a row selected */
+        if (gtk_tree_selection_count_selected_rows (selection) > 0)
+        {
+            GList *llist = NULL;
+            GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
+            /* start a command list */
+            /* grab the selected songs */
+            list = gtk_tree_selection_get_selected_rows (selection, &model);
+            /* grab the last song that is selected */
+            llist = g_list_last(list);
+            /* remove every selected song one by one */
+            if(llist){
+                int id;
+                char *path = NULL;
+                int *indices = gtk_tree_path_get_indices((GtkTreePath *)llist->data);
+                int length = GMPC_MPDDATA_MODEL(model)->num_rows;
+                GList *liter = g_list_first(paste_list);
+                id = indices[0];
+                while(liter)
+                {
+                    path = liter->data;
+                    mpd_database_playlist_list_add(connection, pl_path, path);
+                    mpd_database_playlist_move(connection, pl_path,length, id+1); 
+                    length++;
+                    liter = g_list_next(liter);
+                }
+            }
+            /* free list */
+            g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
+            g_list_free (list);
+        }else{
+            GList *liter = g_list_last(paste_list);;
+            while(liter)
+            {
+                char *path = liter->data;
+                mpd_database_playlist_list_add(connection, pl_path, path);
+                liter = g_list_previous(liter);
+            }
 
-static void playlist_editor_browser_init()
+        }
+        gtk_tree_selection_unselect_all(selection);
+        g_free(pl_path);
+    }
+
+}
+static void playlist_editor_paste_before_songs(GtkTreeView *tree, GList *paste_list)
+{
+	gchar *pl_path = NULL;
+	GList *it,*list = gtk_icon_view_get_selected_items(GTK_ICON_VIEW(playlist_editor_icon_view));
+	for(it = g_list_first(list);it; it = g_list_next(it))
+	{
+		GtkTreePath *path = it->data;
+		GtkTreeIter iter;
+		if(gtk_tree_model_get_iter(GTK_TREE_MODEL(playlist_editor_store), &iter, path) && !pl_path)
+		{
+			gtk_tree_model_get(GTK_TREE_MODEL(playlist_editor_store), &iter, PL_NAME, &pl_path, -1);
+		}
+	}
+	g_list_foreach(list, (GFunc)gtk_tree_path_free, NULL);
+	g_list_free(list);
+    if(pl_path)
+    {
+
+        /* grab the selection from the tree */
+        GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(tree));
+
+        /* check if where connected */
+        /* see if there is a row selected */
+        if (gtk_tree_selection_count_selected_rows (selection) > 0)
+        {
+            GList *llist = NULL;
+            GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
+            /* start a command list */
+            /* grab the selected songs */
+            list = gtk_tree_selection_get_selected_rows (selection, &model);
+            /* grab the last song that is selected */
+            llist = g_list_first(list);
+            /* remove every selected song one by one */
+            if(llist){
+                int id;
+                int *indices = gtk_tree_path_get_indices((GtkTreePath *)llist->data);
+                int length = GMPC_MPDDATA_MODEL(model)->num_rows;
+                GList *liter = g_list_first(paste_list);
+                id = indices[0];
+
+                while(liter)
+                {
+                    char *path = liter->data;
+                    mpd_database_playlist_list_add(connection, pl_path, path);
+                    mpd_database_playlist_move(connection, pl_path,length, id); 
+                    /* length one longer */
+                    length++;
+                    liter = g_list_next(liter);
+                }
+            }
+            /* free list */
+            g_list_foreach (list, (GFunc) gtk_tree_path_free, NULL);
+            g_list_free (list);
+        }else{
+            GList *liter = g_list_last(paste_list);
+            while(liter)
+            {
+                char *path = liter->data;
+                mpd_database_playlist_list_add(connection, pl_path, path);
+                liter = g_list_previous(liter);
+            }
+
+        }
+
+        gtk_tree_selection_unselect_all(selection);
+        g_free(pl_path);
+    }
+}
+static void playlist_editor_row_activated(GtkTreeView *tree, GtkTreePath *path, GtkTreeViewColumn *column, gpointer user_data)
+{
+    GtkTreeModel *model = gtk_tree_view_get_model(tree);
+    GtkTreeIter iter;
+    if(gtk_tree_model_get_iter(model,&iter,path))
+    {
+        mpd_Song *song;
+        gtk_tree_model_get(model, &iter, MPDDATA_MODEL_COL_MPDSONG,&song, -1); 
+        if(song->file)
+        {
+            play_path(song->file);
+        }
+    }
+}
+
+static void playlist_editor_browser_init(void)
 {
     GtkWidget *tree = NULL;
     GtkWidget *sw = NULL;
+    int pos = 0;
     /* */
     playlist_editor_browser = gtk_vpaned_new();
     /** browser */
     sw = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
     gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
-    gtk_widget_set_size_request(sw, -1, 100);	
+
+    pos = cfg_get_single_value_as_int_with_default(config, "playlist-plugin", "pane-pos",100);
     gtk_paned_add1(GTK_PANED(playlist_editor_browser), sw);
 
+    gtk_paned_set_position(GTK_PANED(playlist_editor_browser), pos);
+
     /* icon view*/
     playlist_editor_store = gtk_list_store_new(PL_NUM_COLS, G_TYPE_STRING, G_TYPE_STRING,GDK_TYPE_PIXBUF);
 
@@ -700,11 +871,23 @@ static void playlist_editor_browser_init()
 
 
     playlist_editor_song_tree = tree = gmpc_mpddata_treeview_new("playlist-browser",FALSE, GTK_TREE_MODEL(playlist_editor_list_store));
+    gmpc_mpddata_treeview_enable_click_fix(GMPC_MPDDATA_TREEVIEW(playlist_editor_song_tree));
+
+    /* Copy paste routines */
+    g_signal_connect(G_OBJECT(tree), "cut", G_CALLBACK(playlist_editor_cut_songs), NULL);
+    g_signal_connect(G_OBJECT(tree), "paste-after", G_CALLBACK(playlist_editor_paste_after_songs), NULL);
+    g_signal_connect(G_OBJECT(tree), "paste-before", G_CALLBACK(playlist_editor_paste_before_songs), NULL);
+
+
+    g_signal_connect(G_OBJECT(tree), "row-activated", G_CALLBACK(playlist_editor_row_activated), NULL);
+
+
+
     gtk_container_add(GTK_CONTAINER(sw), tree);
     gtk_tree_view_set_reorderable(GTK_TREE_VIEW(tree), TRUE);
 
     g_signal_connect(G_OBJECT(tree), "button-release-event", G_CALLBACK(playlist_editor_key_released), NULL);
-    g_signal_connect(G_OBJECT(tree), "button-press-event", G_CALLBACK(playlist_editor_key_pressed), NULL);
+    g_signal_connect(G_OBJECT(tree), "key-press-event", G_CALLBACK(playlist_editor_key_pressed), NULL);
     g_object_ref(playlist_editor_browser);
 
     gtk_widget_show_all(playlist_editor_browser);
@@ -769,9 +952,16 @@ void playlist_editor_right_mouse(GtkWidget *menu, void (*add_to_playlist)(GtkWid
         {
             if(data->type ==  MPD_DATA_TYPE_PLAYLIST)
             {
+                
                 sitem = gtk_image_menu_item_new_with_label(data->playlist->path);
-                gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(sitem), 
-                        gtk_image_new_from_icon_name("media-playlist", GTK_ICON_SIZE_MENU));
+                if(g_utf8_collate(data->playlist->path, _("Favorites"))== 0)
+                {
+                    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(sitem), 
+                            gtk_image_new_from_icon_name("emblem-favorite", GTK_ICON_SIZE_MENU));
+                }else{
+                    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(sitem), 
+                            gtk_image_new_from_icon_name("media-playlist", GTK_ICON_SIZE_MENU));
+                }
                 g_object_set_data_full(G_OBJECT(sitem),"playlist", g_strdup(data->playlist->path), g_free);
                 gtk_menu_shell_append(GTK_MENU_SHELL(smenu), sitem);
                 g_signal_connect(G_OBJECT(sitem), "activate", G_CALLBACK(add_to_playlist), NULL);
@@ -813,6 +1003,15 @@ static void playlist_editor_activate(GtkWidget *item, gpointer data)
 }
 
 
+static void favorites_add_current_song(void)
+{
+  mpd_Song *song = mpd_playlist_get_current_song(connection);
+  if(song->file) 
+  {
+      mpd_database_playlist_list_add(connection, _("Favorites"), song->file); 
+      playlist3_show_error_message(_("Added playing song to favorites list."), ERROR_INFO);
+  }
+}
 static int playlist_editor_add_go_menu(GtkWidget *menu)
 {
     GtkWidget *item = NULL;
@@ -829,6 +1028,14 @@ static int playlist_editor_add_go_menu(GtkWidget *menu)
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
     g_signal_connect(G_OBJECT(item), "activate", 
             G_CALLBACK(playlist_editor_activate), NULL);
+    /** */
+    item = gtk_image_menu_item_new_with_label(_("Add Current Song to favorites"));
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), gtk_image_new_from_icon_name("emblem-favorite", GTK_ICON_SIZE_MENU));
+    gtk_widget_add_accelerator(GTK_WIDGET(item), "activate", gtk_menu_get_accel_group(GTK_MENU(menu)), GDK_Return, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
+    g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(favorites_add_current_song), NULL);
+
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+
     return 1;
 }
 
@@ -845,5 +1052,9 @@ static void playlist_editor_save_myself(void)
             gtk_tree_path_free(path);
         }
     }
+    if(playlist_editor_browser) {
+        int pos = gtk_paned_get_position(GTK_PANED(playlist_editor_browser));
+        cfg_set_single_value_as_int(config, "playlist-plugin", "pane-pos", pos);
+    }
 }
 
