@@ -1,3 +1,22 @@
+/* Gnome Music Player Client (GMPC)
+ * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Project homepage: http://gmpc.wikia.com/
+ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
+
 /**
  * TODO:
  * $ If disconnected, don't destroy all tag browsers, just clear the first one, if connected refill the first one
@@ -12,17 +31,15 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <libmpd/debug_printf.h>
-#include "plugin.h"
 #include "main.h"
 #include "misc.h"
 #include "gmpc-mpddata-treeview.h"
 #include "gmpc-mpddata-model.h"
 #ifdef USE_SYSTEM_LIBSEXY
-#include <sexy-icon-entry.h>
-#else
-#include "sexy-icon-entry.h"
+#include <libsexy/sexy-icon-entry.h>
 #endif
 #include <libmpd/libmpd-internal.h>
+#include "playlist3.h"
 #include "playlist3-playlist-editor.h"
 
 /**
@@ -84,7 +101,7 @@ gmpcPlBrowserPlugin tag2_browser_plugin ={
 gmpcPlugin tag2_plug = {
 	.name = N_("Tag based browser"),
 	.version = {0,15,0},
-	.plugin_type = GMPC_PLUGIN_PL_BROWSER,
+	.plugin_type = GMPC_PLUGIN_PL_BROWSER|GMPC_INTERNALL,
 	.init = tag2_init,
 	.destroy = tag2_destroy,
 	.save_yourself = tag2_save_myself,
@@ -196,7 +213,7 @@ typedef struct _tag_element{
 /**
  * Get/Set enabled
  */
-int tag2_get_enabled()
+int tag2_get_enabled(void)
 {
 	return cfg_get_single_value_as_int_with_default(config, "tag2-plugin", "enable", TRUE);
 }
@@ -214,7 +231,7 @@ static void tag2_set_enabled(int enabled)
 /* 
  * Destroy the browser
  */
-static void tag2_destroy()
+static void tag2_destroy(void)
 {
 	/* clear all the browsers */	
 	g_list_foreach(tag2_ht, (GFunc)tag2_destroy_browser, NULL);
@@ -224,6 +241,8 @@ static void tag2_destroy()
 
 static void tag2_destroy_tag(tag_element *te)
 {
+    /* Part of inconsistent update hack */
+    g_idle_remove_by_data(te);
 	if(te->timeout)
 		g_source_remove(te->timeout);
 	gtk_widget_destroy(te->vbox);
@@ -256,6 +275,7 @@ static gboolean tag2_song_list_button_press_event(GtkWidget *but, GdkEventButton
                 return TRUE;
             }else{
                 gtk_tree_selection_unselect_path(sel, path);
+                gtk_tree_path_free(path);
                 return TRUE;
             }
 
@@ -323,7 +343,7 @@ static void tag2_browser_header_information(GtkWidget *item, tag_element *te)
         {
             gchar *value = NULL;
             gtk_tree_model_get(te->model, &iter, MPDDATA_MODEL_COL_SONG_TITLE, &value, -1);
-            if(te->type == MPD_TAG_ITEM_ARTIST)
+            if(te->type == MPD_TAG_ITEM_ARTIST || te->type == MPD_TAG_ITEM_ALBUM_ARTIST)
             {
                 info2_activate();
                 info2_fill_artist_view(value);
@@ -381,7 +401,7 @@ static gboolean tag2_browser_button_release_event(GtkTreeView *tree, GdkEventBut
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(tag2_browser_replace_selected), te);
 
-        if(te->type == MPD_TAG_ITEM_ARTIST || (te->type == MPD_TAG_ITEM_ALBUM && gmpc_mpddata_model_get_request_artist(GMPC_MPDDATA_MODEL(te->model)) != NULL))
+        if(te->type == MPD_TAG_ITEM_ARTIST || te->type == MPD_TAG_ITEM_ALBUM_ARTIST || (te->type == MPD_TAG_ITEM_ALBUM && gmpc_mpddata_model_get_request_artist(GMPC_MPDDATA_MODEL(te->model)) != NULL))
         {
             item = gtk_image_menu_item_new_from_stock(GTK_STOCK_INFO,NULL);
             gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
@@ -397,10 +417,38 @@ static gboolean tag2_browser_button_release_event(GtkTreeView *tree, GdkEventBut
 	return FALSE;
 }
 
+static gboolean tag2_key_entry_key_press_event(GtkWidget *entry, GdkEventKey *event, tag_element *te)
+{
+	const gchar *text = gtk_entry_get_text(GTK_ENTRY(entry));
+
+	if(strlen(text) == 0)
+	{
+		if(event->keyval == GDK_BackSpace || event->keyval == GDK_Escape)
+		{
+			gtk_widget_hide(te->sentry);
+			gtk_widget_grab_focus(te->tree);
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
 static int  tag2_key_release_event(GtkTreeView *tree, GdkEventKey *event,tag_element *te)
 {
-    if((event->state&(GDK_CONTROL_MASK|GDK_MOD1_MASK)) == 0 && 
-            ((event->keyval >= GDK_space && event->keyval <= GDK_z)))
+	guint32 uc;
+	if((event->state&GDK_CONTROL_MASK) != 0 && event->keyval == GDK_f)
+	{
+        gtk_widget_grab_focus(te->sentry);      
+       	gtk_widget_show(te->sentry); 
+		return FALSE;
+	}/*
+	else if(event->keyval == GDK_Escape){
+        gtk_entry_set_text(GTK_ENTRY(te->sentry),"");
+		gtk_widget_hide(te->sentry);
+		return FALSE;
+	}*/
+	else  if((event->state&(GDK_CONTROL_MASK|GDK_MOD1_MASK)) == 0 && 
+/*            ((event->keyval >= GDK_space && event->keyval <= GDK_z)))*/
+		(uc = gdk_keyval_to_unicode(event->keyval)))
     {
         char data[2];
         data[0] = (char)gdk_keyval_to_unicode(event->keyval);
@@ -415,6 +463,20 @@ static int  tag2_key_release_event(GtkTreeView *tree, GdkEventKey *event,tag_ele
 
 }
 
+/**
+ * This is a hack here, for one purpose.
+ * When mpd database changes, it could hapening that when updating a sel change on column A
+ * that during the update, the selection of column B changes, what causes another change in column A
+ * So you get recursive updates.
+ * To avoid this, we "delay" the update callbacks to after the update on column A is completely handled.
+ * TODO: This needs to be handled nicer. 
+ */
+static void tag2_changed(GtkTreeSelection *sel2, tag_element *te);
+static gboolean tag2_changed_delayed(gpointer data)
+{
+    tag2_changed(NULL, (tag_element *)data);
+    return FALSE;
+}
 static void tag2_changed(GtkTreeSelection *sel2, tag_element *te)
 {
     int nfilter = cfg_get_single_value_as_int_with_default(config, "tag2-plugin", "don't filter", 0);
@@ -424,6 +486,13 @@ static void tag2_changed(GtkTreeSelection *sel2, tag_element *te)
     int not_to_update = te->index;
     int artist_set = FALSE;
     GList *tel = g_list_first(browser->tag_lists);
+    static int working = 0;
+    if(working){
+	    g_idle_add(tag2_changed_delayed,te);
+	    return;
+    }
+    working = 1;
+
 	/* Clear songs list */
     /* clear the depending browsers */
 	while(tel)
@@ -440,7 +509,7 @@ static void tag2_changed(GtkTreeSelection *sel2, tag_element *te)
 	while(tel)
     {
         te = tel->data;
-        if(te->index != not_to_update)
+        if(te->index != not_to_update && mpd_server_tag_supported(connection,te->type))
         {
             GtkTreeSelection *sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(te->tree));
             GtkTreeIter iter;
@@ -461,20 +530,23 @@ static void tag2_changed(GtkTreeSelection *sel2, tag_element *te)
                     sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(te3->tree));	
                     if(gtk_tree_selection_get_selected(sel, &(te3->model), &iter))
                     {	
-                        gchar *value;
-                        gtk_tree_model_get(te3->model, &iter, MPDDATA_MODEL_COL_SONG_TITLE, &value, -1);
-                        if(!nfilter || te3->index < te->index)
-                            mpd_database_search_add_constraint(connection, te3->type, (value)?value:"");
-                        if(te3->index < (te->index) && !artist_seen)
-                        {           
-                            if(te3->type == MPD_TAG_ITEM_ARTIST)
-                            {
-                                artist = g_strdup(value);
-                                artist_seen = 1;
+                        if(mpd_server_tag_supported(connection,te3->type))
+                        {
+                            gchar *value;
+                            gtk_tree_model_get(te3->model, &iter, MPDDATA_MODEL_COL_SONG_TITLE, &value, -1);
+                            if(!nfilter || te3->index < te->index)
+                                mpd_database_search_add_constraint(connection, te3->type, (value)?value:"");
+                            if(te3->index < (te->index) && !artist_seen)
+                            {           
+                                if(te3->type == MPD_TAG_ITEM_ARTIST || te3->type == MPD_TAG_ITEM_ALBUM_ARTIST)
+                                {
+                                    artist = g_strdup(value);
+                                    artist_seen = 1;
+                                }
                             }
-                        }
 
-                        g_free(value);
+                            g_free(value);
+                        }
                     }
                 }
                 first =first->next;
@@ -582,7 +654,7 @@ static void tag2_changed(GtkTreeSelection *sel2, tag_element *te)
     if(found)
         data = mpd_database_search_commit(connection);                                          		
     gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(gtk_tree_view_get_model(browser->tag_songlist)), data);     
-
+    working = 0;
 }
 
 static void tag2_destroy_browser(tag_browser *browser, gpointer user_data) 
@@ -636,19 +708,22 @@ static void tag2_songlist_combo_box_changed(GtkComboBox *box, tag_element *te)
     /* clear the list, makes changing the tree layout better.*/
     gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(te->model), NULL);
     /* clear all settings */
-    gtk_cell_layout_clear_attributes(GTK_CELL_LAYOUT(te->column), te->image_renderer);
-    if(te->type == MPD_TAG_ITEM_ARTIST ||te->type == MPD_TAG_ITEM_ALBUM)
+    if(te->image_renderer)
     {
-        int size = cfg_get_single_value_as_int_with_default(config, "gmpc-mpddata-model", "icon-size", 32);
-	    gtk_tree_view_column_add_attribute(te->column,te->image_renderer, "pixbuf", MPDDATA_MODEL_META_DATA);
-        gtk_cell_renderer_set_fixed_size(te->image_renderer, size,size);
-    }
-    else
-    {
-        int width, height;
-        gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
-        gtk_cell_renderer_set_fixed_size(te->image_renderer, width,height);
-	    gtk_tree_view_column_add_attribute(te->column,te->image_renderer, "icon-name", MPDDATA_MODEL_COL_ICON_ID);
+        gtk_cell_layout_clear_attributes(GTK_CELL_LAYOUT(te->column), te->image_renderer);
+        if(te->type == MPD_TAG_ITEM_ARTIST || te->type == MPD_TAG_ITEM_ALBUM_ARTIST ||te->type == MPD_TAG_ITEM_ALBUM)
+        {
+            int size = cfg_get_single_value_as_int_with_default(config, "gmpc-mpddata-model", "icon-size", 32);
+            gtk_tree_view_column_add_attribute(te->column,te->image_renderer, "pixbuf", MPDDATA_MODEL_META_DATA);
+            gtk_cell_renderer_set_fixed_size(te->image_renderer, size,size);
+        }
+        else
+        {
+            int width, height;
+            gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
+            gtk_cell_renderer_set_fixed_size(te->image_renderer, width,height);
+            gtk_tree_view_column_add_attribute(te->column,te->image_renderer, "icon-name", MPDDATA_MODEL_COL_ICON_ID);
+        }
     }
 
     /* index starts at 1 */
@@ -673,9 +748,14 @@ static void tag2_songlist_combo_box_changed(GtkComboBox *box, tag_element *te)
         }
         else
         {
-            mpd_database_search_field_start(connection, te->type);
-            data = mpd_database_search_commit(connection);
-            gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(te->model), data);
+            if(mpd_server_tag_supported(connection,te->type))
+            {
+                mpd_database_search_field_start(connection, te->type);
+                data = mpd_database_search_commit(connection);
+                gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(te->model), data);
+            }else {
+                gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(te->model), NULL);
+            }
         }
     }
 
@@ -708,12 +788,13 @@ static gboolean tag2_sentry_changed_real(tag_element *te)
     return FALSE;
 }
 
-static void tag2_sentry_changed(SexyIconEntry *entry, tag_element *te)
+static void tag2_sentry_changed(GtkWidget *entry, tag_element *te)
 {
     gtk_widget_show(te->sentry);
     if(te->timeout)
         g_source_remove(te->timeout);
-    te->timeout = g_timeout_add_seconds(1, (GSourceFunc)tag2_sentry_changed_real, te);
+    te->timeout = g_timeout_add(250, (GSourceFunc)tag2_sentry_changed_real, te);
+//    tag2_sentry_changed_real(te);
 }
 
 
@@ -739,11 +820,17 @@ static void tag2_songlist_clear_selection(GtkWidget *button, tag_browser *browse
     {
         MpdData *data;
         tag_element *te = iter->data;
-        mpd_database_search_field_start(connection, te->type);
-        
-        data = mpd_database_search_commit(connection);
+
         gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(te->model), NULL);
-        gmpc_mpddata_model_set_mpd_data_slow(GMPC_MPDDATA_MODEL(te->model), data);
+        if(mpd_server_tag_supported(connection,te->type))
+        {
+            mpd_database_search_field_start(connection, te->type);
+
+            data = mpd_database_search_commit(connection);
+            gmpc_mpddata_model_set_mpd_data_slow(GMPC_MPDDATA_MODEL(te->model), data);
+        }else 
+            gmpc_mpddata_model_set_mpd_data_slow(GMPC_MPDDATA_MODEL(te->model), NULL);
+        gmpc_mpddata_model_set_request_artist(GMPC_MPDDATA_MODEL(te->model), NULL);
     }
                                     		
     gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(gtk_tree_view_get_model(browser->tag_songlist)), NULL);     
@@ -768,8 +855,14 @@ static void tag2_songlist_add_tag(tag_browser *browser,const gchar *name, int ty
 	te->index 	= g_list_length(browser->tag_lists);
 	te->model 	= (GtkTreeModel *) gmpc_mpddata_model_new();
 	te->combo   = gtk_combo_box_new_with_model(tags_store);
+
+#ifdef USE_SYSTEM_LIBSEXY
     te->sentry  = sexy_icon_entry_new(); 
-	te->sw 		= gtk_scrolled_window_new(NULL,NULL);
+#else
+    te->sentry  = gtk_entry_new();
+#endif
+    g_signal_connect_swapped(G_OBJECT(te->sentry), "activate", G_CALLBACK(tag2_sentry_changed_real), te);
+    te->sw 		= gtk_scrolled_window_new(NULL,NULL);
     te->vbox    = gtk_vbox_new(FALSE, 6);
 	te->tree 	= gtk_tree_view_new_with_model(GTK_TREE_MODEL(te->model));	
 	te->browser = browser;
@@ -786,8 +879,14 @@ static void tag2_songlist_add_tag(tag_browser *browser,const gchar *name, int ty
     /* entry */
     gtk_widget_set_no_show_all(te->sentry, TRUE);
     gtk_widget_hide(te->sentry);
+
+#ifdef USE_SYSTEM_LIBSEXY
     sexy_icon_entry_add_clear_button(SEXY_ICON_ENTRY(te->sentry));
+#endif
+
     g_signal_connect(G_OBJECT(te->sentry), "changed", G_CALLBACK(tag2_sentry_changed), te);
+    g_signal_connect(G_OBJECT(te->sentry), "key-press-event", G_CALLBACK(tag2_key_entry_key_press_event), te);
+
     gtk_box_pack_start(GTK_BOX(te->vbox), te->sentry,FALSE, TRUE, 0);
 
     g_signal_connect(G_OBJECT(te->tree), "key-press-event", G_CALLBACK(tag2_key_release_event), te);
@@ -806,26 +905,29 @@ static void tag2_songlist_add_tag(tag_browser *browser,const gchar *name, int ty
 	te->column = column = gtk_tree_view_column_new();
 	
 	gtk_tree_view_column_set_title(column, name);
-
-	te->image_renderer = renderer = gtk_cell_renderer_pixbuf_new();
-
-    gtk_tree_view_column_pack_start(column, renderer, FALSE);
-    if(te->type == MPD_TAG_ITEM_ARTIST ||te->type == MPD_TAG_ITEM_ALBUM)
+    te->image_renderer = NULL;
+    if(cfg_get_single_value_as_int_with_default(config, "tag2-plugin","show-image-column", 1))
     {
-        int size = cfg_get_single_value_as_int_with_default(config, "gmpc-mpddata-model", "icon-size", 32);
+        te->image_renderer = renderer = gtk_cell_renderer_pixbuf_new();
 
-        gtk_tree_view_column_add_attribute(column, renderer, "pixbuf", MPDDATA_MODEL_META_DATA);
-        gtk_cell_renderer_set_fixed_size(renderer, size,size);
-        gtk_tree_view_column_set_sizing(column , GTK_TREE_VIEW_COLUMN_FIXED);
-    }
-    else
-    {
-        int width, height;
-        gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
-        gtk_cell_renderer_set_fixed_size(te->image_renderer, width,height);
-	    gtk_tree_view_column_add_attribute(column, renderer, "icon-name", MPDDATA_MODEL_COL_ICON_ID);
-        gtk_tree_view_column_set_sizing(column , GTK_TREE_VIEW_COLUMN_FIXED);
+        gtk_tree_view_column_pack_start(column, renderer, FALSE);
+        if(te->type == MPD_TAG_ITEM_ARTIST || te->type == MPD_TAG_ITEM_ALBUM_ARTIST ||te->type == MPD_TAG_ITEM_ALBUM)
+        {
+            int size = cfg_get_single_value_as_int_with_default(config, "gmpc-mpddata-model", "icon-size", 32);
+
+            gtk_tree_view_column_add_attribute(column, renderer, "pixbuf", MPDDATA_MODEL_META_DATA);
+            gtk_cell_renderer_set_fixed_size(renderer, size,size);
+        }
+        else
+        {
+            int width, height;
+            gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, &width, &height);
+            gtk_cell_renderer_set_fixed_size(te->image_renderer, width,height);
+            gtk_tree_view_column_add_attribute(column, renderer, "icon-name", MPDDATA_MODEL_COL_ICON_ID);
+        }
     }
+
+    gtk_tree_view_column_set_sizing(column , GTK_TREE_VIEW_COLUMN_FIXED);
     gtk_tree_view_set_fixed_height_mode(GTK_TREE_VIEW(te->tree), TRUE);
     renderer = gtk_cell_renderer_text_new();
 	gtk_tree_view_column_pack_start(column, renderer, TRUE);
@@ -917,15 +1019,6 @@ static void tag2_songlist_replace_selected_songs(GtkWidget *bug, tag_browser *br
 	}
 }
 
-/**
- * Calls the column editing function of the GmpcTreeview
- */
-static void tag2_song_list_edit_columns(GtkMenuItem *item, tag_browser *browser)
-{
-	gmpc_mpddata_treeview_edit_columns(GMPC_MPDDATA_TREEVIEW(browser->tag_songlist));
-}
-
-
 /**
  * Handles right mouse release on song list
  */
@@ -984,13 +1077,6 @@ static gboolean tag2_song_list_button_release_event(GtkTreeView *tree,
 		}
 
         gmpc_mpddata_treeview_right_mouse_intergration(GMPC_MPDDATA_TREEVIEW(tree), GTK_MENU(menu));
-        /* Edit columns */
-		item = gtk_image_menu_item_new_with_label(("Edit Columns"));
-		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item),
-				gtk_image_new_from_stock(GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU));
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-		g_signal_connect(G_OBJECT(item), "activate",
-				G_CALLBACK(tag2_song_list_edit_columns), browser);
      		/* popup */
 		gtk_widget_show_all(GTK_WIDGET(menu));
 		gtk_menu_popup(GTK_MENU(menu), NULL, NULL,NULL, NULL, 0, event->time);
@@ -1179,9 +1265,14 @@ static void tag2_connection_changed_foreach(tag_browser *browser, gpointer userd
 		{
 			MpdData *data;
             tag_element *te2 = NULL;
-			mpd_database_search_field_start(connection, te->type);
-			data = mpd_database_search_commit(connection);
-			gmpc_mpddata_model_set_mpd_data_slow(GMPC_MPDDATA_MODEL(te->model), data);
+            if(mpd_server_tag_supported(connection,te->type))
+            {
+                mpd_database_search_field_start(connection, te->type);
+                data = mpd_database_search_commit(connection);
+                gmpc_mpddata_model_set_mpd_data_slow(GMPC_MPDDATA_MODEL(te->model), data);
+            }else {
+                gmpc_mpddata_model_set_mpd_data_slow(GMPC_MPDDATA_MODEL(te->model), NULL);
+            }
 
 			te2 = g_malloc0(sizeof(*te2));
 			te2->index = -1;
@@ -1212,6 +1303,12 @@ static void tag2_status_changed(MpdObj *mi, ChangedStatusType what, gpointer dat
         if(tag2_ht)
         {
             g_list_foreach(tag2_ht,(GFunc)tag2_connection_changed_foreach, NULL);
+/*            GList *list = g_list_first(tag2_ht);
+            for(;list;list = g_list_next(list))
+            {
+                tag2_songlist_clear_selection(NULL, list->data); 
+            }
+            */
         }
     }
 }
@@ -1413,9 +1510,14 @@ static void tag2_pref_column_add(GtkWidget *but, GtkComboBox *box)
 				MpdData *data;
 				tag_element *te2 = giter->data;
 				/* update the content */	
-				mpd_database_search_field_start(connection, te2->type);
-				data = mpd_database_search_commit(connection);                        			
-				gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(te2->model), data);	
+                if(mpd_server_tag_supported(connection,te2->type))
+                {
+                    mpd_database_search_field_start(connection, te2->type);
+                    data = mpd_database_search_commit(connection);                        			
+                    gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(te2->model), data);	
+                }else{
+                    gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(te2->model), NULL);	
+                }
 			}
 		}
 	}
@@ -1738,3 +1840,5 @@ static int tag2_browser_key_press_event ( GtkWidget *mw, GdkEventKey *event, int
 
     return 0;
 }
+
+/* vim: set noexpandtab ts=4 sw=4 sts=4 tw=120*/
