@@ -1,26 +1,28 @@
-/*
- *Copyright (C) 2004-2008 Qball Cow <qball@sarine.nl>
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
+/* Gnome Music Player Client (GMPC)
+ * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Project homepage: http://gmpc.wikia.com/
+ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- */
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
 
 #include <gtk/gtk.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include <glib.h>
+#include <glib/gstdio.h>
 #ifndef WIN32
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -58,17 +60,18 @@ typedef struct _config_obj
 	GMutex *lock;
 	config_node *root;
 	int total_size;
+    guint save_timeout;
 } _config_obj;
 
-static void __int_cfg_set_single_value_as_string(config_obj *cfg, char *class, char *sclass, char *ssclass, char *key, char *value);
-static void cfg_save_real(config_obj *);
+static void __int_cfg_set_single_value_as_string(config_obj *cfg, const char *class, const char *sclass, const char *ssclass, const char *key, const char *value);
+static gboolean cfg_save_real(config_obj *);
 static void __int_cfg_remove_node(config_obj *, config_node *);
-static config_node *cfg_get_class_multiple(config_obj *cfg, config_node *root, char *class);
+static config_node *cfg_get_class_multiple(config_obj *cfg, config_node *root, const char *class);
 static void __int_cfg_do_special_cleanup(config_obj *cfg, config_node *node);
-static config_node *cfg_add_class(config_obj *, config_node *,char *);
+static config_node *cfg_add_class(config_obj *, config_node *,const char *);
 static config_node *cfg_new_node(void);
 static void cfg_add_child(config_node *, config_node *);
-#define cfg_save(a) debug_printf(DEBUG_INFO, "Save triggered");cfg_save_real(a)
+static void cfg_save_delayed(config_obj *cfg);
 
 static void cfg_open_parse_file(config_obj *cfgo, FILE *fp)
 {
@@ -228,10 +231,11 @@ config_obj *cfg_open(gchar *url)
 	cfgo->url = g_strdup(url);
 	cfgo->root = NULL;
 	cfgo->total_size = sizeof(config_obj)+sizeof(&cfgo->lock)+strlen(cfgo->url);
+    cfgo->save_timeout = 0;
 
 	if(g_file_test(cfgo->url, G_FILE_TEST_EXISTS))
 	{
-		FILE *fp = fopen(cfgo->url, "r");
+		FILE *fp = g_fopen(cfgo->url, "r");
 		if(fp)
 		{
 			cfg_open_parse_file(cfgo, fp);
@@ -250,7 +254,7 @@ void cfg_close(config_obj *cfgo)
 	{
 		return;
 	}
-    cfg_save(cfgo);
+    cfg_save_real(cfgo);
 	debug_printf(DEBUG_INFO,"Closing config '%s' with %i bytes allocated\n", cfgo->url, cfgo->total_size);
 	g_mutex_lock(cfgo->lock);
 	if(cfgo->url != NULL)
@@ -266,7 +270,7 @@ void cfg_close(config_obj *cfgo)
 	g_free(cfgo);
 	cfgo = NULL;
 }
-static config_node *cfg_new_node()
+static config_node *cfg_new_node(void)
 {
 	config_node *newnode = g_slice_new(config_node);
 	newnode->type = TYPE_CATEGORY;
@@ -275,16 +279,15 @@ static config_node *cfg_new_node()
 	newnode->prev = NULL;                                 
 	newnode->parent = NULL;
 	newnode->value = NULL;
-	return newnode;
+    newnode->children  = NULL;
+    return newnode;
 }
-static config_node *cfg_add_class(config_obj *cfg,config_node *parent, char *class)
+static config_node *cfg_add_class(config_obj *cfg,config_node *parent, const char *class)
 {
 	config_node *newnode = cfg_new_node();
 	newnode->type = TYPE_CATEGORY;
 	newnode->name = g_strdup(class);
-	newnode->value = NULL;
     newnode->parent = parent;
-	newnode->children  = NULL;
 	cfg->total_size += sizeof(config_node)+strlen(class);
     if(!parent)
     {
@@ -387,17 +390,24 @@ static void cfg_save_category(config_obj *cfg, config_node *node, FILE *fp)
 	}
 }
 
-static void cfg_save_real(config_obj *cfgo)
+static gboolean cfg_save_real(config_obj *cfgo)
 {
-	if(cfgo == NULL)
-	{
-		return;
+    if(cfgo == NULL)
+    {
+        return FALSE;
 	}
+
+    if(cfgo->save_timeout) {
+        cfgo->save_timeout = 0;
+    }else {
+        return FALSE;
+    }
+    debug_printf(DEBUG_INFO, "Save triggered:%s", cfgo->url);
     debug_printf(DEBUG_INFO,"Saving config file: %s (%i bytes)", cfgo->url, cfgo->total_size);
 	if(cfgo->root != NULL)
 	{
-		FILE *fp = fopen(cfgo->url, "w");
-		if(!fp) return;
+		FILE *fp = g_fopen(cfgo->url, "w");
+		if(!fp) return FALSE;
         fputs("2\n", fp);
 		cfg_save_category(cfgo,cfgo->root, fp);	
 		fclose(fp);
@@ -406,15 +416,15 @@ static void cfg_save_real(config_obj *cfgo)
 #endif
 
 	}
-	return;
+	return FALSE;
 }
 
-static config_node *cfg_get_class(config_obj *cfg, char *class)
+static config_node *cfg_get_class(config_obj *cfg, const char *class)
 {
     return cfg_get_class_multiple(cfg, NULL, class);
 }
 
-static config_node *cfg_get_class_multiple(config_obj *cfg, config_node *root, char *class)
+static config_node *cfg_get_class_multiple(config_obj *cfg, config_node *root, const char *class)
 {
 	config_node *node = NULL;
     if(root != NULL)
@@ -439,7 +449,7 @@ static config_node *cfg_get_class_multiple(config_obj *cfg, config_node *root, c
 	return NULL;
 }
 
-static config_node *cfg_get_single_value(config_obj *cfg, char *class, char *sclass, char *ssclass, char *key)
+static config_node *cfg_get_single_value(config_obj *cfg, const char *class, const char *sclass, const char *ssclass, const char *key)
 {
 	/* take children */
 	config_node *cur = NULL;
@@ -474,7 +484,7 @@ static config_node *cfg_get_single_value(config_obj *cfg, char *class, char *scl
 	}
 }
 */
-static char * __int_cfg_get_single_value_as_string(config_obj *cfg, char *class, char * sclass, char *ssclass, char *key)
+static char * __int_cfg_get_single_value_as_string(config_obj *cfg, const char *class, const char * sclass, const char *ssclass, const char *key)
 {
 	config_node *cur = cfg_get_single_value(cfg, class, sclass, ssclass,key);
 	if(cur != NULL)
@@ -487,18 +497,18 @@ static char * __int_cfg_get_single_value_as_string(config_obj *cfg, char *class,
 	return NULL;
 }
 
-char * cfg_get_single_value_as_string(config_obj *cfg, char *class, char *key)
+char * cfg_get_single_value_as_string(config_obj *cfg, const char *class, const char *key)
 {
     return cfg_get_single_value_as_string_mm(cfg, class, NULL, NULL,key);
 }
 
 
-char * cfg_get_single_value_as_string_with_default(config_obj *cfg, char *class, char *key , char *def)
+char * cfg_get_single_value_as_string_with_default(config_obj *cfg, const char *class, const char *key , const char *def)
 {
     return cfg_get_single_value_as_string_with_default_mm(cfg, class, NULL, NULL,key,def);
 }
 
-static int __int_cfg_get_single_value_as_int_mm(config_obj *cfg, char *class, char *sclass, char *ssclass, char *key)
+static int __int_cfg_get_single_value_as_int_mm(config_obj *cfg, const char *class, const char *sclass, const char *ssclass, const char *key)
 {
 	config_node *cur =NULL;
 	int retv = CFG_INT_NOT_DEFINED;
@@ -514,7 +524,7 @@ static int __int_cfg_get_single_value_as_int_mm(config_obj *cfg, char *class, ch
 
 	return retv; 
 }
-int cfg_get_single_value_as_int(config_obj *cfg, char *class, char *key)
+int cfg_get_single_value_as_int(config_obj *cfg, const char *class, const char *key)
 {
 	int retv = 0;
 	g_mutex_lock(cfg->lock);
@@ -522,11 +532,11 @@ int cfg_get_single_value_as_int(config_obj *cfg, char *class, char *key)
 	g_mutex_unlock(cfg->lock);
 	return retv;
 }
-void cfg_set_single_value_as_int(config_obj *cfg, char *class, char *key, int value)
+void cfg_set_single_value_as_int(config_obj *cfg, const char *class, const char *key, int value)
 {
     cfg_set_single_value_as_int_mm(cfg,class,NULL, NULL, key,value);
 }
-static void __int_cfg_set_single_value_as_int(config_obj *cfg, char *class, char *sclass, char *ssclass,char *key, int value)
+static void __int_cfg_set_single_value_as_int(config_obj *cfg, const char *class, const char *sclass, const char *ssclass,const char *key, int value)
 {
 	gchar *	temp = NULL;
 	temp  = g_strdup_printf("%i",value);
@@ -534,12 +544,12 @@ static void __int_cfg_set_single_value_as_int(config_obj *cfg, char *class, char
 	cfg_free_string(temp);
 }
 
-int cfg_get_single_value_as_int_with_default(config_obj *cfg, char *class, char *key, int def)
+int cfg_get_single_value_as_int_with_default(config_obj *cfg, const char *class, const char *key, int def)
 {
     return  cfg_get_single_value_as_int_with_default_mm(cfg, class,NULL, NULL, key, def);
 }
 /* float */
-static float __int_cfg_get_single_value_as_float(config_obj *cfg, char *class,char *sclass, char *ssclass, char *key)
+static float __int_cfg_get_single_value_as_float(config_obj *cfg, const char *class,const char *sclass, const char *ssclass, const char *key)
 {
 	float result = 0;
 	config_node *cur =NULL;
@@ -554,7 +564,7 @@ static float __int_cfg_get_single_value_as_float(config_obj *cfg, char *class,ch
 	return result;
 }
 
-float cfg_get_single_value_as_float(config_obj *cfg, char *class, char *key)
+float cfg_get_single_value_as_float(config_obj *cfg, const char *class, const char *key)
 {
 	float retv = 0;
 	g_mutex_lock(cfg->lock);
@@ -564,7 +574,7 @@ float cfg_get_single_value_as_float(config_obj *cfg, char *class, char *key)
 }
 
 
-float cfg_get_single_value_as_float_mm(config_obj *cfg, char *class, char *sclass, char *ssclass, char *key)
+float cfg_get_single_value_as_float_mm(config_obj *cfg, const char *class, const char *sclass, const char *ssclass, const char *key)
 {
 	float retv = 0;
 	g_mutex_lock(cfg->lock);
@@ -573,7 +583,7 @@ float cfg_get_single_value_as_float_mm(config_obj *cfg, char *class, char *sclas
 	return retv;
 }
 
-void cfg_set_single_value_as_float(config_obj *cfg, char *class, char *key, float value)
+void cfg_set_single_value_as_float(config_obj *cfg, const char *class, const char *key, float value)
 {
 	char *value1 = NULL;
 	g_mutex_lock(cfg->lock);
@@ -582,7 +592,7 @@ void cfg_set_single_value_as_float(config_obj *cfg, char *class, char *key, floa
 	cfg_free_string(value1);
 	g_mutex_unlock(cfg->lock);
 }
-float cfg_get_single_value_as_float_with_default(config_obj *cfg, char *class, char *key, float def)
+float cfg_get_single_value_as_float_with_default(config_obj *cfg, const char *class, const char *key, float def)
 {
 	float retv = 0;
 	g_mutex_lock(cfg->lock);
@@ -655,7 +665,7 @@ static void __int_cfg_remove_node(config_obj *cfg, config_node *node)
 	}
 	g_slice_free(config_node,node);
 }
-void cfg_del_single_value_mm(config_obj *cfg, char *class,char *sclass, char *ssclass, char *key)
+void cfg_del_single_value_mm(config_obj *cfg, const char *class,const char *sclass, const char *ssclass, const char *key)
 {
 	config_node *node = NULL;
 	g_mutex_lock(cfg->lock);
@@ -663,16 +673,17 @@ void cfg_del_single_value_mm(config_obj *cfg, char *class,char *sclass, char *ss
 	if(node != NULL)
 	{
 		__int_cfg_remove_node(cfg,node);
-		//cfg_save(cfg);
+		cfg_save_delayed(cfg);
+        debug_printf(DEBUG_INFO, "triggered save delay: del: %s, %s", class, key);
 	}
 	g_mutex_unlock(cfg->lock);
 }
-void cfg_del_single_value(config_obj *cfg, char *class, char *key)
+void cfg_del_single_value(config_obj *cfg, const char *class, const char *key)
 {
     cfg_del_single_value_mm(cfg,class,NULL, NULL, key);
 }
 
-void cfg_remove_class(config_obj *cfg, char *class)
+void cfg_remove_class(config_obj *cfg, const char *class)
 {
 	config_node *node = NULL;
 	if(cfg == NULL || class == NULL)
@@ -684,10 +695,11 @@ void cfg_remove_class(config_obj *cfg, char *class)
 	{
 		__int_cfg_remove_node(cfg, node);
 	}
-	//cfg_save(cfg);
-	g_mutex_unlock(cfg->lock);
+	cfg_save_delayed(cfg);
+    debug_printf(DEBUG_INFO, "triggered save delay: del: %s", class);
+    g_mutex_unlock(cfg->lock);
 }
-static void __int_cfg_set_single_value_as_string(config_obj *cfg, char *class, char *sclass, char *ssclass, char *key, char *value)
+static void __int_cfg_set_single_value_as_string(config_obj *cfg, const char *class, const char *sclass, const char *ssclass, const char *key, const char *value)
 {
 	config_node *newnode = cfg_get_single_value(cfg,class,sclass, ssclass, key);
 	if(newnode == NULL)
@@ -734,10 +746,11 @@ static void __int_cfg_set_single_value_as_string(config_obj *cfg, char *class, c
 	}
 	newnode->value = g_strdup(value);	
 	cfg->total_size += strlen(value);
-	//cfg_save(cfg);
+	cfg_save_delayed(cfg);
+    debug_printf(DEBUG_INFO, "triggered save delay: set: %s,%s -> %s", class,key,value);
 }
 
-void cfg_set_single_value_as_string(config_obj *cfg, char *class, char *key, char *value)
+void cfg_set_single_value_as_string(config_obj *cfg, const char *class, const char *key, const char *value)
 {
 	g_mutex_lock(cfg->lock);
 	__int_cfg_set_single_value_as_string(cfg, class,NULL, NULL, key,value);
@@ -747,7 +760,7 @@ void cfg_set_single_value_as_string(config_obj *cfg, char *class, char *key, cha
 /* multiple values */
 
 
-static config_node *cfg_get_multiple_value(config_obj *cfg, char *class, char *key, char* id)
+static config_node *cfg_get_multiple_value(config_obj *cfg, const char *class, const char *key, const char* id)
 {
 	config_node *node = cfg_get_single_value(cfg,class,NULL, NULL,key);
 	if(node == NULL){
@@ -772,7 +785,7 @@ static config_node *cfg_get_multiple_value(config_obj *cfg, char *class, char *k
 	return NULL;
 }
 
-void cfg_del_multiple_value(config_obj *cfg, char *class, char *key,char *id)
+void cfg_del_multiple_value(config_obj *cfg, const char *class, const char *key,const char *id)
 {
 	config_node *cur = NULL;
 	g_mutex_lock(cfg->lock);
@@ -780,12 +793,12 @@ void cfg_del_multiple_value(config_obj *cfg, char *class, char *key,char *id)
 	if(cur != NULL)
 	{
 		__int_cfg_remove_node(cfg,cur);
-		//cfg_save(cfg);
+		cfg_save_delayed(cfg);
 	}
 	g_mutex_unlock(cfg->lock);
 }
 
-void cfg_set_multiple_value_as_string(config_obj *cfg, char *class, char *key, char *id, char *value)
+void cfg_set_multiple_value_as_string(config_obj *cfg, const char *class, const char *key, const char *id, const char *value)
 {
 	config_node *cur = NULL;
 	g_mutex_lock(cfg->lock);
@@ -803,7 +816,7 @@ void cfg_set_multiple_value_as_string(config_obj *cfg, char *class, char *key, c
 
 		cur->value = g_strdup(value);
 		cfg->total_size += strlen(cur->value);
-		//cfg_save(cfg);
+		cfg_save_delayed(cfg);
 	}
 	else {
 		config_node *node = cfg_get_single_value(cfg,class,NULL, NULL,key);
@@ -827,12 +840,12 @@ void cfg_set_multiple_value_as_string(config_obj *cfg, char *class, char *key, c
 		cur->value = g_strdup(value);
 		cfg->total_size+=sizeof(config_node)+strlen(id)+strlen(value);;
 		cfg_add_child(node,cur);
-		//cfg_save(cfg);
+		cfg_save_delayed(cfg);
 	}
 	g_mutex_unlock(cfg->lock);
 }
 
-conf_mult_obj * cfg_get_multiple_as_string(config_obj *cfg, char *class, char *key)
+conf_mult_obj * cfg_get_multiple_as_string(config_obj *cfg, const char *class, const char *key)
 {
 	conf_mult_obj *list = NULL;
 	config_node *cur = NULL;
@@ -908,7 +921,7 @@ conf_mult_obj *cfg_get_class_list(config_obj *data)
 	return list;
 }
 
-conf_mult_obj *cfg_get_key_list(config_obj *data,char *class)
+conf_mult_obj *cfg_get_key_list(config_obj *data,const char *class)
 {
 	conf_mult_obj *list = NULL;
 	config_node *root = NULL;
@@ -994,7 +1007,7 @@ void cfg_do_special_cleanup(config_obj *cfg)
 
 	g_mutex_lock(cfg->lock);
 	__int_cfg_do_special_cleanup(cfg, NULL);
-	cfg_save(cfg);
+	cfg_save_delayed(cfg);
 	g_mutex_unlock(cfg->lock);
 }
 
@@ -1005,13 +1018,13 @@ void cfg_do_special_cleanup(config_obj *cfg)
 /**
  * String 
  */
-void cfg_set_single_value_as_string_mm(config_obj *cfg, char *class,char *sclass, char *ssclass, char *key, char *value)
+void cfg_set_single_value_as_string_mm(config_obj *cfg, const char *class,const char *sclass, const char *ssclass, const char *key, const char *value)
 {
 	g_mutex_lock(cfg->lock);
 	__int_cfg_set_single_value_as_string(cfg, class,sclass,ssclass, key,value);
 	g_mutex_unlock(cfg->lock);
 }
-char * cfg_get_single_value_as_string_mm(config_obj *cfg, char *class,char *sclass, char *ssclass, char *key)
+char * cfg_get_single_value_as_string_mm(config_obj *cfg, const char *class,const char *sclass, const char *ssclass, const char *key)
 {
 	char *retv = NULL;
 	g_mutex_lock(cfg->lock);
@@ -1021,7 +1034,7 @@ char * cfg_get_single_value_as_string_mm(config_obj *cfg, char *class,char *scla
 }
 
 
-char * cfg_get_single_value_as_string_with_default_mm(config_obj *cfg, char *class, char *sclass, char *ssclass, char *key , char *def)
+char * cfg_get_single_value_as_string_with_default_mm(config_obj *cfg, const char *class, const char *sclass, const char *ssclass, const char *key , const char *def)
 {
 	char *retv = NULL;
    	g_mutex_lock(cfg->lock);
@@ -1037,7 +1050,7 @@ char * cfg_get_single_value_as_string_with_default_mm(config_obj *cfg, char *cla
 /**
  * Int 
  */
-void cfg_set_single_value_as_int_mm(config_obj *cfg, char *class,char *sclass, char *ssclass, char *key, int value)
+void cfg_set_single_value_as_int_mm(config_obj *cfg, const char *class,const char *sclass, const char *ssclass, const char *key, int value)
 {
 	gchar *	temp = NULL;
 	g_mutex_lock(cfg->lock);
@@ -1046,7 +1059,7 @@ void cfg_set_single_value_as_int_mm(config_obj *cfg, char *class,char *sclass, c
 	cfg_free_string(temp);
 	g_mutex_unlock(cfg->lock);
 }
-int cfg_get_single_value_as_int_with_default_mm(config_obj *cfg, char *class, char *sclass, char *ssclass, char *key, int def)
+int cfg_get_single_value_as_int_with_default_mm(config_obj *cfg, const char *class, const char *sclass, const char *ssclass, const char *key, int def)
 {
 	int retv = CFG_INT_NOT_DEFINED;
 	g_mutex_lock(cfg->lock);		
@@ -1059,3 +1072,12 @@ int cfg_get_single_value_as_int_with_default_mm(config_obj *cfg, char *class, ch
 	g_mutex_unlock(cfg->lock);		
 	return retv;
 }
+
+static void cfg_save_delayed(config_obj *cfg)
+{
+    if(cfg->save_timeout) {
+        g_source_remove(cfg->save_timeout);
+        cfg->save_timeout = 0;
+    }
+    cfg->save_timeout = g_timeout_add_seconds(5, (GSourceFunc)cfg_save_real, cfg);
+}
