@@ -1,29 +1,26 @@
-/*
- *Copyright (C) 2004-2008 Qball Cow <qball@sarine.nl>
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
+/* Gnome Music Player Client (GMPC)
+ * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Project homepage: http://gmpc.wikia.com/
+
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-
-
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdlib.h>
 #include <string.h>
-#include <glade/glade.h>
 #include <time.h>
 #include "main.h"
 #include "misc.h"
@@ -32,7 +29,7 @@
 
 
 
-GladeXML *xml_esf  = NULL;
+static GtkBuilder *xml_esf  = NULL;
 GtkTextBuffer *buffer= NULL;
 
 void esf_reload(void);
@@ -54,27 +51,27 @@ static void esf_render_example(GtkTextBuffer *buf)
 	song->file = g_strdup(filename);
 	song->time = song_time;
 
-	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml_esf, "ck_artist"))))
+	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(xml_esf, "ck_artist"))))
 	{
 		song->artist = g_strdup(artist);
 	}
-	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml_esf, "ck_album"))))
+	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(xml_esf, "ck_album"))))
 	{
 		song->album = g_strdup(album);
 	}
-	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml_esf, "ck_title"))))
+	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(xml_esf, "ck_title"))))
 	{
 		song->title = g_strdup(title);
 	}
-	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml_esf, "ck_track"))))
+	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(xml_esf, "ck_track"))))
 	{
 		song->track = g_strdup(track);
 	}
-	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml_esf, "ck_date"))))
+	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(xml_esf, "ck_date"))))
 	{
 		song->date = g_strdup(date);
 	}
-	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml_esf, "ck_stream"))))
+	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gtk_builder_get_object(xml_esf, "ck_stream"))))
 	{
 		song->name = g_strdup(stream_name);
 	}
@@ -85,7 +82,7 @@ static void esf_render_example(GtkTextBuffer *buf)
 	format = gtk_text_buffer_get_text(buffer,&start_iter, &stop_iter, FALSE);
 	mpd_song_markup((char *)result_buffer, 1024,format, song);
 
-	gtk_label_set_text(GTK_LABEL(glade_xml_get_widget(xml_esf, "label_example")), (const char *)result_buffer);
+	gtk_label_set_text(GTK_LABEL(gtk_builder_get_object(xml_esf, "label_example")), (const char *)result_buffer);
 	mpd_freeSong(song);
 }
 void esf_reload(void)
@@ -97,12 +94,19 @@ char * edit_song_markup(char *format)
 	char *str_format = NULL;
 	GtkTextIter start_iter, stop_iter;
 	GtkWidget *dialog;
-	char *path = gmpc_get_full_glade_path("gmpc.glade");
-	xml_esf= glade_xml_new(path, "esf_dialog",NULL);
+	GError *error = NULL;
+	char *path = gmpc_get_full_glade_path("preferences-esf-dialog.ui");
+	xml_esf= gtk_builder_new();
+	gtk_builder_add_from_file(xml_esf, path, &error);
+	if(error) {
+		debug_printf(DEBUG_ERROR, error->message);
+		g_error_free(error);
+		error = NULL;
+	}
 	q_free(path);
-	dialog = glade_xml_get_widget(xml_esf, "esf_dialog");
-	buffer= gtk_text_view_get_buffer(GTK_TEXT_VIEW(glade_xml_get_widget(xml_esf, "textview_markup")));
-	glade_xml_signal_autoconnect (xml_esf);
+	dialog = (GtkWidget *) gtk_builder_get_object(xml_esf, "esf_dialog");
+	buffer= gtk_text_view_get_buffer(GTK_TEXT_VIEW(gtk_builder_get_object(xml_esf, "textview_markup")));
+	gtk_builder_connect_signals(xml_esf, NULL);	
 	g_signal_connect(G_OBJECT(buffer), "changed", G_CALLBACK(esf_render_example), NULL);
 	if(format != NULL)
 	{
@@ -118,6 +122,8 @@ char * edit_song_markup(char *format)
 				gtk_text_buffer_get_end_iter(buffer, &stop_iter);
 				str_format = g_strdup(gtk_text_buffer_get_text(buffer,&start_iter, &stop_iter, FALSE));
 			}
+		default:
+			break;
 	}
 
 	gtk_widget_destroy(dialog);
