@@ -1,31 +1,37 @@
+/* Gnome Music Player Client (GMPC)
+ * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Project homepage: http://gmpc.wikia.com/
+ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
+
 #ifndef __MAIN_H__
 
 #define __MAIN_H__
 #include <config.h>
-
+#include <gmpc-version.h>
 #ifdef ENABLE_NLS
-#include <libintl.h>
-#include <locale.h>
-#define _(String) gettext (String)
-#  ifdef gettext_noop
-#    define N_(String) gettext_noop (String)
-#  else
-#    define N_(String) (String)
-#  endif
-#else 
-#define _(String) String
-#define N_(String) String
+#include <glib/gi18n.h>
 #endif
 
 #include "config-defaults.h"
 #include <libmpd/libmpd.h>
 #include <libmpd/libmpdclient.h>
 #include <libmpd/debug_printf.h>
-#include <glade/glade.h>
 #include "config1.h"
 #include "plugin.h"
-#include "metadata.h"
-#include "playlist3.h"
 #include "mpdinteraction.h"
 
 #include "playlist3-messages.h" 
@@ -36,13 +42,13 @@
  */
 #include "gob/gmpc-profiles.h"
 #include "gob/gmpc-connection.h"
-#include "gob/gmpc-metaimage.h"
 #include "gob/gmpc-signals.h"
-#include "gob/gmpc-idle.h"
 
+#include "vala/gmpc-easy-command.h"
+
+extern GmpcEasyCommand *gmpc_easy_command;
 extern GmpcConnection 	*gmpcconn;
 extern GmpcSignals 		*gmpc_signals;
-extern GmpcIdle 		*gmpc_idle;
 extern int 				gmpc_connected;
 extern GtkTreeModel 	*playlist;
 
@@ -55,18 +61,12 @@ char * edit_song_markup(char *format);
 
 void main_quit(void);
 
-/*
- * functions to get patch to different files.
- * This is needed to make the windows port work.
- */
-char *gmpc_get_full_glade_path(char *filename);
-
 /* plugin */
 void plugin_load_dir(gchar *path);
 void plugin_add(gmpcPlugin *plug, int plugin);
 int plugin_get_pos(int id);
 
-void show_error_message(gchar *string, int block);
+void show_error_message(const gchar *string,const int block);
 
 /** Handle status changed */
 void   GmpcStatusChangedCallback(MpdObj *mi, ChangedStatusType what, void *userdata);
@@ -85,8 +85,9 @@ void url_start(void);
 /*
  * functions to get patch to different files.
  * This is needed to make the windows port work.
+ * (misc.c)
  */
-char *gmpc_get_full_image_path(char *filename);
+char *gmpc_get_full_image_path(const char *filename);
 
 /**
  * Help functions
@@ -98,21 +99,22 @@ void info2_activate(void);
 void info2_fill_artist_view(const char *);
 void info2_fill_album_view(const char *,const char *);
 void info2_fill_song_view(mpd_Song *);
+
 /* tray stuff */
 gboolean tray_icon2_get_available(void);
 void tray_icon2_create_tooltip(void);
 
-/* usefull for some stuff */
-extern GladeXML *pl3_xml;
-
+/* tag stuff */
 
+void mpd_interaction_update_supported_tags(void);
 /**
  * Playlist functions 
  */
 
 /* Check if the playlist is fullscreen, this queries the _actual_ state */
 gboolean pl3_window_is_fullscreen(void);
-
+/* easy download */
+void gmpc_easy_async_quit(void);
 
 #ifdef DEBUG_TIMING
 /* Tic Tac system */
