@@ -1,3 +1,22 @@
+/* Gnome Music Player Client (GMPC)
+ * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Project homepage: http://gmpc.wikia.com/
+ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
+
 #include <string.h>
 #include <glib.h>
 #include <glib/gstdio.h>
@@ -14,6 +33,8 @@ GMutex *meta_plugins_lock = NULL;
 gmpcPlugin **meta_plugins = NULL;
 static void meta_data_sort_plugins(void);
 GThread *meta_thread = NULL;
+
+static MetaDataResult meta_data_get_from_cache(mpd_Song *song, MetaDataType type, char **path);
 /**
  * This is queue is used to send commands to the retrieval queue
  */
@@ -51,12 +72,12 @@ static mpd_Song *rewrite_mpd_song(mpd_Song *tsong, MetaDataType type)
     /* If it is not a mpd got song */
     if(tsong->file == NULL )
     {
-        if(type&(META_ALBUM_ART|META_ALBUM_TXT))
+        if(type&(META_ALBUM_ART|META_ALBUM_TXT) && tsong->artist && tsong->album)
         {
             MpdData *data2 = NULL;
             mpd_database_search_start(connection, TRUE);
-            mpd_database_search_add_constraint(connection, MPD_TAG_ITEM_ARTIST, (tsong->artist)?tsong->artist:"");
-            mpd_database_search_add_constraint(connection, MPD_TAG_ITEM_ALBUM,  (tsong->album)?tsong->album:""); 
+            mpd_database_search_add_constraint(connection, MPD_TAG_ITEM_ARTIST, tsong->artist);
+            mpd_database_search_add_constraint(connection, MPD_TAG_ITEM_ALBUM,  tsong->album); 
             data2 = mpd_database_search_commit(connection);
             if(data2)
             {
@@ -64,12 +85,27 @@ static mpd_Song *rewrite_mpd_song(mpd_Song *tsong, MetaDataType type)
                 data2->song = NULL;
                 mpd_data_free(data2);
             }
+            else if(tsong->albumartist)
+            {
+                mpd_database_search_start(connection, TRUE);
+                mpd_database_search_add_constraint(connection, MPD_TAG_ITEM_ALBUM_ARTIST, tsong->albumartist);
+                mpd_database_search_add_constraint(connection, MPD_TAG_ITEM_ALBUM,  tsong->album); 
+                data2 = mpd_database_search_commit(connection);
+                if(data2)
+                {
+                    edited = data2->song;
+                    data2->song = NULL;
+                    mpd_data_free(data2);
+                }
+
+
+            }
         }
-        if(type&(META_ARTIST_ART|META_ARTIST_TXT))
+        if(type&(META_ARTIST_ART|META_ARTIST_TXT) && tsong->artist)
         {
             MpdData *data2 = NULL;
             mpd_database_search_start(connection, TRUE);
-            mpd_database_search_add_constraint(connection, MPD_TAG_ITEM_ARTIST, (tsong->artist)?tsong->artist:"");
+            mpd_database_search_add_constraint(connection, MPD_TAG_ITEM_ARTIST, tsong->artist);
             data2 = mpd_database_search_commit(connection);
             if(data2)
             {
@@ -88,7 +124,14 @@ static mpd_Song *rewrite_mpd_song(mpd_Song *tsong, MetaDataType type)
      */
     if(type&(META_ALBUM_ART|META_ALBUM_TXT))
     {
-        if(edited->album && edited->file)
+        if(edited->albumartist)
+        {
+            if(edited->artist)
+                g_free(edited->artist);
+            edited->artist = g_strdup(edited->albumartist);
+
+        }
+        else if(edited->album && edited->file)
         {
             int i=0;
             MpdData *data2;
@@ -266,6 +309,7 @@ void meta_data_set_cache(mpd_Song *song, MetaDataType type, MetaDataResult resul
         if(strcmp(edited->artist, "Various Artists")!=0)
             meta_data_set_cache_real(song, type, result, path);
     }
+    
     mpd_freeSong(edited);
 }
 
@@ -275,7 +319,7 @@ void meta_data_set_cache(mpd_Song *song, MetaDataType type, MetaDataResult resul
  */
 
 
-MetaDataResult meta_data_get_from_cache(mpd_Song *song, MetaDataType type, char **path)
+static MetaDataResult meta_data_get_from_cache(mpd_Song *song, MetaDataType type, char **path)
 {
 	if(!song)
 	{
@@ -490,7 +534,7 @@ MetaDataResult meta_data_get_from_cache(mpd_Song *song, MetaDataType type, char
 	return META_DATA_FETCHING;	
 }
 
-static void meta_data_retrieve_thread()
+static void meta_data_retrieve_thread(void)
 {
 	meta_thread_data *data = NULL;
 	/**
@@ -570,7 +614,7 @@ static void meta_data_retrieve_thread()
 				if(gmpc_plugin_get_enabled(plug))
 				{
                     debug_printf(DEBUG_INFO, "Query plugin: '%s'", plug->name);
-					data->result = plug->metadata->get_image(data->edited, data->type&META_QUERY_DATA_TYPES, &path);
+					data->result = gmpc_plugin_metadata_get_image(plug,data->edited, data->type&META_QUERY_DATA_TYPES, &path);
 					data->result_path = path;
 				}
 			}
@@ -624,7 +668,8 @@ static gboolean meta_data_handle_results(void)
 		}
 
         if(data->result_path)q_free(data->result_path);
-        mpd_freeSong(data->song);
+        if(data->song)
+            mpd_freeSong(data->song);
         mpd_freeSong(data->edited);
         q_free(data);
 	}
@@ -635,14 +680,13 @@ static gboolean meta_data_handle_results(void)
     test = g_mutex_trylock(meta_processing);
     if(test)
         g_mutex_unlock(meta_processing);
-    gmpc_meta_watcher_queue_size_changed(gmw, q_async_queue_true_length(meta_commands)+!test, num_queries);
     return FALSE;
 }
 
 /**
  * Initialize
  */
-void meta_data_init()
+void meta_data_init(void)
 {
     gchar *url = gmpc_get_covers_path(NULL); 
     if(!g_file_test(url,G_FILE_TEST_IS_DIR)){
@@ -698,7 +742,7 @@ static void meta_data_sort_plugins(void)
         changed=0;
         for(i=0; i< (meta_num_plugins-1);i++)
         {
-            if(meta_plugins[i]->metadata->get_priority() > meta_plugins[i+1]->metadata->get_priority())
+            if(gmpc_plugin_metadata_get_priority(meta_plugins[i]) > gmpc_plugin_metadata_get_priority(meta_plugins[i+1]))
             {
                 gmpcPlugin *temp = meta_plugins[i];
                 changed=1;
@@ -719,18 +763,12 @@ static gboolean meta_data_check_plugin_changed_message(gpointer data)
 	playlist3_show_error_message(_("A new metadata plugin was added, gmpc has purged all failed hits from the cache"), ERROR_INFO);
 	return FALSE;
 }
-void meta_data_check_plugin_changed()
+void meta_data_check_plugin_changed(void)
 {
     int old_amount= cfg_get_single_value_as_int_with_default(config, "metadata", "num_plugins", 0);
     if(old_amount < meta_num_plugins)
     {
-/*
-        GtkWidget *dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
-                _("A new metadata plugin was added, gmpc will now purge all missing metadata from the cache"));
-        g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(gtk_widget_destroy), NULL);
-        gtk_widget_show_all(GTK_WIDGET(dialog));
-*/
-	gtk_init_add(meta_data_check_plugin_changed_message, NULL);
+        gtk_init_add(meta_data_check_plugin_changed_message, NULL);
         meta_data_cleanup();
     }
     if(old_amount != meta_num_plugins)
@@ -855,10 +893,12 @@ MetaDataResult meta_data_get_path(mpd_Song *tsong, MetaDataType type, gchar **pa
          * If the data is know. (and doesn't need fectching) 
          * call the callback and stop
          */
+        
         if(ret != META_DATA_FETCHING)
         {
             /* store it under the original */
             meta_data_set_cache_real(tsong, type, ret, *path);
+
             mpd_freeSong(mtd->edited);
             q_free(mtd);
             return ret;	
@@ -912,7 +952,6 @@ MetaDataResult meta_data_get_path(mpd_Song *tsong, MetaDataType type, gchar **pa
     test = g_mutex_trylock(meta_processing);
     if(test)
         g_mutex_unlock(meta_processing);
-    gmpc_meta_watcher_queue_size_changed(gmw, q_async_queue_true_length(meta_commands)+!test, num_queries);
 
     q_async_queue_push(meta_commands, mtd);
     /** clean reference to pointer, it's now to the other thread */
@@ -924,7 +963,9 @@ MetaDataResult meta_data_get_path(mpd_Song *tsong, MetaDataType type, gchar **pa
 static gchar * strip_invalid_chars(gchar *input)
 {
     int i = 0;
-    int length = strlen(input);
+    int length = 0;
+    g_assert(input != NULL);
+    length = strlen(input);
     if(input == NULL) return NULL;
     for(i=0;i<length;i++)
     {
@@ -955,15 +996,34 @@ gchar * gmpc_get_metadata_filename(MetaDataType  type, mpd_Song *song, char *ext
     gchar *retv= NULL;
     /* home dir */
     const gchar *homedir = g_get_home_dir();
+    g_assert(song->artist != NULL);
     g_assert(type < META_QUERY_DATA_TYPES); 
     {
+        const gchar *charset;
+        GError *error = NULL;
         gchar *filename = NULL, *dirname = NULL;
-        gchar *extention= (type&(META_ALBUM_TXT|META_ARTIST_TXT|META_SONG_TXT))?"txt":((ext == NULL)?"":ext);
+        const gchar *extention= (type&(META_ALBUM_TXT|META_ARTIST_TXT|META_SONG_TXT))?"txt":((ext == NULL)?"":ext);
         g_assert(song->artist != NULL);
 
         /* Convert it so the filesystem likes it */
         /* TODO: Add error checking */
-        dirname = g_filename_from_utf8(song->artist,-1,NULL,NULL,NULL); 
+
+        if (g_get_charset (&charset))
+        {
+            debug_printf(DEBUG_INFO, "Locale is utf-8, just copying");
+            dirname = g_strdup(song->artist);
+        }else{
+            debug_printf(DEBUG_INFO, "Locale is %s converting to UTF-8", charset);
+            dirname = g_convert_with_fallback (song->artist, -1,
+                      charset, "UTF-8",NULL, NULL, NULL, &error);
+        }
+        //dirname = g_filename_from_utf8(song->artist,-1,NULL,NULL,&error); 
+        if(error) {
+            debug_printf(DEBUG_ERROR, "Failed to convert %s to file encoding. '%s'", song->artist, error->message);
+            g_error_free(error);
+            if(dirname) g_free(dirname);
+            dirname = g_strdup("invalid");
+        }
         dirname = strip_invalid_chars(dirname);
         retv = g_build_path(G_DIR_SEPARATOR_S, homedir,METADATA_DIR, dirname,NULL);
         if(g_file_test(retv, G_FILE_TEST_EXISTS) == FALSE) {
@@ -1010,8 +1070,8 @@ static void metadata_pref_priority_changed(GtkCellRenderer *renderer, char *path
         gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &plug, -1);
         if(plug)
         {
-            plug->metadata->set_priority((gint)g_ascii_strtoull(new_text, NULL, 0));
-            gtk_list_store_set(GTK_LIST_STORE(store), &iter, 2, plug->metadata->get_priority(),-1);
+            gmpc_plugin_metadata_set_priority(plug,(gint)g_ascii_strtoull(new_text, NULL, 0));
+            gtk_list_store_set(GTK_LIST_STORE(store), &iter, 2, gmpc_plugin_metadata_get_priority(plug),-1);
             meta_data_sort_plugins();
         }
     }
@@ -1072,12 +1132,14 @@ static void metadata_construct_pref_pane(GtkWidget *container)
     for(i=0; i< meta_num_plugins;i++)
     {
         GtkTreeIter iter;
-        gtk_list_store_insert_with_values(store, &iter, -1, 
-            0, meta_plugins[i],
-            1, gmpc_plugin_get_name(meta_plugins[i]),
-            2, meta_plugins[i]->metadata->get_priority(),
-            -1);
-
+        if(gmpc_plugin_get_enabled(meta_plugins[i]))
+        {
+            gtk_list_store_insert_with_values(store, &iter, -1, 
+                    0, meta_plugins[i],
+                    1, gmpc_plugin_get_name(meta_plugins[i]),
+                    2, gmpc_plugin_metadata_get_priority(meta_plugins[i]),
+                    -1);
+        }
     }
 
     label = gtk_label_new("Plugins are evaluated from low priority to high");
