@@ -1,29 +1,29 @@
-/*
- * Copyright (C) 2004-2008 Qball Cow <qball@sarine.nl>
+/* Gnome Music Player Client (GMPC)
+ * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
  * Borrowed from Lee Willis <lee@leewillis.co.uk> that
  * Borrowed heavily from code by Jan Arne Petersen <jpetersen@uni-bonn.de>
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
+ * This projects' homepage is: http://gmpc.wikia.com/
+ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- */
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
+
 #include <config.h>
 #ifdef ENABLE_MMKEYS
 
 #include <stdio.h>
 #include "mm-keys.h"
-#include "eggcellrendererkeys.h"
 
 static void mmkeys_class_init (MmKeysClass *klass);
 static void mmkeys_init       (MmKeys      *object);
@@ -65,10 +65,11 @@ enum {
 	MM_VOLUME_DOWN,
 	MM_SHOW_NOTIFICATION,
     MM_TOGGLE_MUTE,
+    MM_SHOW_EASY_COMMAND,
 	LAST_SIGNAL
 };
 
-char *keynames[LAST_SIGNAL] = {
+const char *keynames[LAST_SIGNAL] = {
 	N_("PlayPause"),	/** MM_PLAYPAUSE */
 	N_("Next"),	 	/** MM_NEXT*/
 	N_("Previous"),		/** MM_PREV */
@@ -84,6 +85,7 @@ char *keynames[LAST_SIGNAL] = {
 	N_("Volume Down"),	/** MM_VOLUME_DOWN */
 	N_("Show song"),	/** MM_SHOW_NOTIFICATION */
     N_("Toggle Mute"),  /** MM_TOGGLE_MUTE */
+    N_("Show easy command entry"),
 };	
 
 static GObjectClass *parent_class;
@@ -246,6 +248,14 @@ static void mmkeys_class_init (MmKeysClass *klass)
 				g_cclosure_marshal_VOID__INT,	
 				G_TYPE_NONE, 1, G_TYPE_INT); 	
 
+	signals[MM_SHOW_EASY_COMMAND] = 
+		g_signal_new ("mm_show_easy_command",
+				G_TYPE_FROM_CLASS (klass),
+				G_SIGNAL_RUN_LAST,
+				0,
+				NULL, NULL,
+				g_cclosure_marshal_VOID__INT,	
+				G_TYPE_NONE, 1, G_TYPE_INT); 	
 
 
 }
@@ -255,12 +265,32 @@ static void mmkeys_finalize (GObject *object)
 	parent_class->finalize (G_OBJECT(object));
 }
 
+static gchar *  convert_keysym_state_to_string (
+        guint keysym,
+        GdkModifierType mask,
+        guint keycode)
+{
+    gchar *name;
+
+    name = gtk_accelerator_name (keysym, mask);
+    if (keysym == 0)
+    {
+        gchar *tmp;
+
+        tmp = name;
+        name = g_strdup_printf ("%s0x%02x", tmp, keycode);
+        g_free (tmp);
+    }
+
+    return name;
+} 
+
 static void mmkeys_init (MmKeys *object)
 {
 	GdkDisplay *display;
 	GdkScreen *screen;
 	GdkWindow *root;
-	guint i, j;
+	gint i, j;
 	int keycode = 0;
 	int anyKeybindsFailed = FALSE;
 	int anyDuplicatesFound = FALSE;
@@ -338,11 +368,9 @@ static void mmkeys_init (MmKeys *object)
 		{
 			if (keyerror[i] && keycodes[i] != 0)
 			{
-				gchar *rawkeysym = egg_virtual_accelerator_name (
+				gchar *keysym = convert_keysym_state_to_string(
 					XKeycodeToKeysym(GDK_DISPLAY(), keycodes[i], 0),
-					keycodes[i], masks[i]);
-				gchar *keysym = g_markup_escape_text(rawkeysym, -1);
-				q_free (rawkeysym);
+					masks[i],keycodes[i] );
 				g_string_append_printf( message,
 					"\t%s: %s\n",
 					_(keynames[i]), keysym );
@@ -415,7 +443,7 @@ static int grab_mmkey (int key_code, unsigned int mask, GdkWindow *root)
 
 static GdkFilterReturn filter_mmkeys (GdkXEvent *xevent, GdkEvent *event, gpointer data)
 {
-	int i;
+	unsigned int i;
 	XEvent *xev;
 	XKeyEvent *key;
 	unsigned int keystate;
@@ -429,7 +457,7 @@ static GdkFilterReturn filter_mmkeys (GdkXEvent *xevent, GdkEvent *event, gpoint
 	keystate = key->state & ~(Mod2Mask | Mod5Mask | LockMask);
 	for(i=0; i < LAST_SIGNAL;i++)
 	{
-		if(keycodes[i] == key->keycode && masks[i] == keystate )
+		if(keycodes[i] == (int)(key->keycode) && masks[i] == keystate )
 		{
 			g_signal_emit (data, signals[i], 0, 0);
 			debug_printf(DEBUG_INFO, "%s pressed", keynames[i]);
@@ -483,7 +511,7 @@ void grab_key(int key, int keycode, unsigned int mask)
 	GdkDisplay *display;
 	GdkScreen *screen;
 	GdkWindow *root;
-	guint i;
+	gint i;
 	display = gdk_display_get_default ();
 
 	/* remove old key */
@@ -526,7 +554,7 @@ void grab_key(int key, int keycode, unsigned int mask)
  */
 void mmkeys_pref_destroy(GtkWidget *container);
 void mmkeys_pref_construct(GtkWidget *container);
-GladeXML *mmkeys_pref_xml = NULL;
+static GtkBuilder *mmkeys_pref_xml = NULL;
 
 gmpcPrefPlugin mmkeys_gpp = {
 	.construct      = mmkeys_pref_construct,
@@ -590,16 +618,14 @@ accel_edited_callback (GtkCellRendererText *cell,
 		if (i == key)
 			continue;
 		if (hardware_keycode != 0 &&
-			keycodes[i] == hardware_keycode &&
+			keycodes[i] == (int)hardware_keycode &&
 			masks[i] == mask)
 		{
 			gchar *message;
-			gchar *rawkeysym = egg_virtual_accelerator_name (
+			gchar *keysym =  convert_keysym_state_to_string (
 				XKeycodeToKeysym(GDK_DISPLAY(), hardware_keycode, 0),
-				hardware_keycode, mask);
-			gchar *keysym = g_markup_escape_text(rawkeysym, -1);
-			q_free (rawkeysym);
-			message = g_strdup_printf( _("<b>Duplicate mapping detected</b>\n\n"
+				mask,hardware_keycode);
+			message = g_strdup_printf( _("Duplicate mapping detected\n\n"
 					"%s is already mapped to %s"),
 					keysym, _(keynames[i]) );
 			q_free (keysym);
@@ -623,11 +649,10 @@ accel_edited_callback (GtkCellRendererText *cell,
 	if( keyerror[key] )
 	{
 		gchar *message;
-		gchar *rawkeysym = egg_virtual_accelerator_name (
+		gchar *keysym = convert_keysym_state_to_string (
 			XKeycodeToKeysym(GDK_DISPLAY(), keycodes[key], 0),
-			keycodes[key], masks[key]);
-		gchar *keysym = g_markup_escape_text(rawkeysym, -1);
-		q_free (rawkeysym);
+			masks[key],keycodes[key] );
+
 		message = g_strdup_printf(
 			_("Could not grab multimedia key:\n\n"
 			"\t%s: %s\n\n"
@@ -639,18 +664,11 @@ accel_edited_callback (GtkCellRendererText *cell,
 	}
 }
 
-
-
-
-
-
-
-
 void mmkeys_pref_destroy(GtkWidget *container)
 {
 	if(mmkeys_pref_xml)
 	{
-		GtkWidget *vbox = glade_xml_get_widget(mmkeys_pref_xml, "mmkeys-vbox");
+		GtkWidget *vbox = (GtkWidget *)gtk_builder_get_object(mmkeys_pref_xml, "mmkeys-vbox");
 		gtk_container_remove(GTK_CONTAINER(container),vbox);
 		g_object_unref(mmkeys_pref_xml);
 		mmkeys_pref_xml = NULL;
@@ -659,13 +677,14 @@ void mmkeys_pref_destroy(GtkWidget *container)
 
 void mmkeys_pref_construct(GtkWidget *container)
 {
-	gchar *path = gmpc_get_full_glade_path("gmpc.glade");
-	mmkeys_pref_xml = glade_xml_new(path, "mmkeys-vbox",NULL);
+	gchar *path = gmpc_get_full_glade_path("preferences-mmkeys.ui");
+	mmkeys_pref_xml = gtk_builder_new();
+    gtk_builder_add_from_file(mmkeys_pref_xml, path, NULL);
 	q_free(path);
 	if(mmkeys_pref_xml)
 	{
 		int i=0;
-		GtkWidget *vbox = glade_xml_get_widget(mmkeys_pref_xml, "mmkeys-vbox");
+		GtkWidget *vbox =(GtkWidget *) gtk_builder_get_object(mmkeys_pref_xml, "mmkeys-vbox");
 		GtkTreeViewColumn *column = NULL;
 		GtkListStore *store = gtk_list_store_new(MM_STORE_COUNT,
 			G_TYPE_STRING,	/* MM_STORE_KEYNAME	*/
@@ -676,21 +695,22 @@ void mmkeys_pref_construct(GtkWidget *container)
 			G_TYPE_STRING	/* MM_STORE_FOREGROUND	*/
 			);
 		GtkCellRenderer *rend =gtk_cell_renderer_text_new();
-		gtk_tree_view_set_model(GTK_TREE_VIEW (glade_xml_get_widget(mmkeys_pref_xml, "mmkeys-tree")), GTK_TREE_MODEL(store));
+		gtk_tree_view_set_model(GTK_TREE_VIEW (gtk_builder_get_object(mmkeys_pref_xml, "mmkeys-tree")), GTK_TREE_MODEL(store));
 
 		column = gtk_tree_view_column_new();
 		gtk_tree_view_column_pack_start(column, rend, TRUE);
 		gtk_tree_view_column_add_attribute(column, rend,
 			"text", MM_STORE_KEYNAME);
 		gtk_tree_view_column_set_title(column, _("Action"));
-		gtk_tree_view_append_column(GTK_TREE_VIEW (glade_xml_get_widget(mmkeys_pref_xml, "mmkeys-tree")), column);
+		gtk_tree_view_append_column(GTK_TREE_VIEW (gtk_builder_get_object(mmkeys_pref_xml, "mmkeys-tree")), column);
 
-		rend =  egg_cell_renderer_keys_new ();
+		rend =  gtk_cell_renderer_accel_new();
 		column = gtk_tree_view_column_new ();
 
-		/*		g_object_set (G_OBJECT (rend), "accel_mode", EGG_CELL_RENDERER_KEYS_MODE_X);*/
-		egg_cell_renderer_keys_set_accel_mode(EGG_CELL_RENDERER_KEYS(rend), EGG_CELL_RENDERER_KEYS_MODE_GTK);
-		g_object_set (G_OBJECT (rend), "editable", TRUE, NULL);
+		g_object_set (G_OBJECT (rend),
+                "editable", TRUE,
+                "accel-mode", GTK_CELL_RENDERER_ACCEL_MODE_OTHER,
+                NULL);
 
 		g_signal_connect (G_OBJECT (rend),
 				"accel_edited",
@@ -706,11 +726,11 @@ void mmkeys_pref_construct(GtkWidget *container)
 		gtk_tree_view_column_set_title(column, _("Shortcut"));
 		gtk_tree_view_column_set_attributes (column, rend,
 				"keycode",	MM_STORE_KEYCODE,
-				"accel_mask",	MM_STORE_MASK,
+				"accel_mods",	MM_STORE_MASK,
 				"accel_key",	MM_STORE_KEYVAL,
 				"foreground",	MM_STORE_FOREGROUND,
 				NULL);
-		gtk_tree_view_append_column (GTK_TREE_VIEW (glade_xml_get_widget(mmkeys_pref_xml, "mmkeys-tree")), column);
+		gtk_tree_view_append_column (GTK_TREE_VIEW (gtk_builder_get_object(mmkeys_pref_xml, "mmkeys-tree")), column);
 
 		gtk_container_add(GTK_CONTAINER(container),vbox);
 		for(i=0;i< LAST_SIGNAL;i++)
