@@ -1,3 +1,22 @@
+/* Gnome Music Player Client (GMPC)
+ * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Project homepage: http://gmpc.wikia.com/
+ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
+
 #include <string.h>
 #include <glib.h>
 #include <gtk/gtk.h>
@@ -54,6 +73,10 @@ static int plugin_validate(gmpcPlugin *plug)
             debug_printf(DEBUG_ERROR, "%s: plugin_type&GMPC_PLUGIN_PL_BROWSER && plugin->browser != NULL Failed",plug->name);
             return FALSE;
         }
+        if(plug->browser->cat_key_press != NULL)
+        {
+            debug_printf(DEBUG_ERROR, "Plugin %s implements a cat_key_press event handler that is deprecated", plug->name);
+        }
     }
     if(plug->plugin_type&GMPC_PLUGIN_META_DATA)
     {
@@ -272,35 +295,222 @@ gboolean gmpc_plugin_get_enabled(gmpcPlugin *plug)
     }
     return TRUE;
 }
+void gmpc_plugin_set_enabled(gmpcPlugin *plug, gboolean enabled)
+{
+    if(plug->set_enabled)
+    {
+        plug->set_enabled(enabled);
+    }
+}
 
 gchar *gmpc_plugin_get_data_path(gmpcPlugin *plug)
 {
 #ifdef WIN32
-    debug_printf(DEBUG_INFO, "path: %s\n", plug->path);
-    return g_strdup(plug->path);
+    gchar *url = g_win32_get_package_installation_directory_of_module(NULL);
+    gchar *retv = g_build_path(G_DIR_SEPARATOR_S, url, "share", "gmpc", "plugins", NULL);
+    return retv;//g_strdup(plug->path);
 #else
     gchar *url = NULL;
-    gchar **items = g_strsplit(plug->path, G_DIR_SEPARATOR_S, -1);
-    int i = 0;
-    int has_lib = 0;
-    /* count number of items, and check for the lib entry */
-    for(i = 0; items && items[i];i++) {
-        if(strcmp(items[i], "lib")==0) {
-            g_free(items[i]);
-            items[i] = g_strdup("share");
-            has_lib = 1;
-        }
-    }
-    if(has_lib == 1) {
-        gchar *temp = g_build_pathv(G_DIR_SEPARATOR_S, items); 
-        url = g_strdup_printf("%c%s%c", G_DIR_SEPARATOR, temp, G_DIR_SEPARATOR);
-        g_free(temp);
-    }else {
-        /* Ok it is a homedir */
+    gchar *homedir = gmpc_get_user_path(""); 
+    if(strncmp(plug->path,homedir ,strlen(homedir)) == 0)
+    {
         url = g_strdup(plug->path);
+    }else{
+       /* Ok it is a homedir */
+        url = g_build_path(G_DIR_SEPARATOR_S, PACKAGE_DATA_DIR, "gmpc","plugins",NULL);
     }
-    g_strfreev(items);
-    debug_printf(DEBUG_INFO, "path: %s\n",url);
+
+    g_free(homedir);
     return url;
 #endif
 }
+
+void gmpc_plugin_mpd_connection_changed(gmpcPlugin *plug, MpdObj *mi, int connected, gpointer data)
+{
+    g_assert(plug != NULL);
+    if(plug->mpd_connection_changed != NULL)
+    {
+        plug->mpd_connection_changed(mi,connected,data);
+    }
+}
+
+gboolean gmpc_plugin_is_browser(gmpcPlugin *plug)
+{
+    return ((plug->plugin_type&GMPC_PLUGIN_PL_BROWSER) != 0);
+}
+void gmpc_plugin_browser_unselected(gmpcPlugin *plug, GtkWidget *container)
+{
+    if(gmpc_plugin_is_browser(plug)) {
+        g_assert(plug->browser != NULL);
+        g_assert(plug->browser->unselected != NULL);
+        plug->browser->unselected(container);
+    }
+}
+void gmpc_plugin_browser_selected(gmpcPlugin *plug, GtkWidget *container)
+{
+    if(gmpc_plugin_is_browser(plug)) {
+        g_assert(plug->browser != NULL);
+        g_assert(plug->browser->selected != NULL);
+        plug->browser->selected(container);
+    }
+}
+
+void gmpc_plugin_browser_add(gmpcPlugin *plug, GtkWidget *cat_tree)
+{
+    if(gmpc_plugin_is_browser(plug)) {
+        g_assert(plug->browser != NULL);
+        if(plug->browser->add)
+        {
+            plug->browser->add(cat_tree);
+        }
+    }
+}
+
+int gmpc_plugin_browser_cat_right_mouse_menu(gmpcPlugin *plug, GtkWidget *menu, int type, GtkWidget *tree, GdkEventButton *event)
+{
+    if(gmpc_plugin_is_browser(plug)) {
+        g_assert(plug->browser != NULL);
+        if(plug->browser->cat_right_mouse_menu != NULL)
+        {
+            return plug->browser->cat_right_mouse_menu(menu,type,tree,event);
+        }
+    }
+    return 0;
+}
+
+int gmpc_plugin_browser_key_press_event(gmpcPlugin *plug, GtkWidget *mw, GdkEventKey *event, int type)
+{
+    if(gmpc_plugin_is_browser(plug)) {
+        g_assert(plug->browser != NULL);
+        if(plug->browser->key_press_event != NULL)
+        {
+            return plug->browser->key_press_event(mw,event,type);
+        }
+    }
+    return 0;
+}
+
+int gmpc_plugin_browser_add_go_menu(gmpcPlugin *plug, GtkWidget *menu)
+{
+    if(gmpc_plugin_is_browser(plug)) {
+        g_assert(plug->browser != NULL);
+        if(plug->browser->add_go_menu != NULL)
+        {
+            return plug->browser->add_go_menu(menu);
+        }
+    }
+    return 0;
+}
+
+int gmpc_plugin_browser_song_list_option_menu(gmpcPlugin *plug, GmpcMpdDataTreeview *tree, GtkMenu *menu) 
+{
+    if(gmpc_plugin_is_browser(plug))
+    {
+        g_assert(plug->browser != NULL);
+        if(plug->browser->song_list_option_menu)
+        {
+            return plug->browser->song_list_option_menu(tree, menu);
+        }
+    }
+    return 0;
+}
+
+gboolean gmpc_plugin_browser_has_integrate_search(gmpcPlugin *plug)
+{
+    if(gmpc_plugin_is_browser(plug))
+    {
+        return plug->browser->integrate_search != NULL;
+    }
+    return FALSE;
+}
+MpdData *gmpc_plugin_browser_integrate_search(gmpcPlugin *plug, const int search_field, const gchar * query, GError **error)
+{
+    if(!gmpc_plugin_browser_has_integrate_search(plug)) return NULL;
+    return plug->browser->integrate_search(search_field,query,error); 
+}
+
+gboolean gmpc_plugin_browser_integrate_search_field_supported(gmpcPlugin *plug, const int search_field)
+{
+    if(!gmpc_plugin_browser_has_integrate_search(plug)) return FALSE;
+	if(plug->browser->integrate_search_field_supported == NULL) return TRUE;
+    return plug->browser->integrate_search_field_supported(search_field); 
+}
+
+gboolean gmpc_plugin_has_preferences(gmpcPlugin *plug)
+{
+    return (plug->pref != NULL);
+}
+
+void gmpc_plugin_preferences_construct(gmpcPlugin *plug,GtkWidget *wid)
+{
+    if(gmpc_plugin_has_preferences(plug))
+    {
+        g_assert(plug->pref != NULL);
+        g_assert(plug->pref->construct);
+        plug->pref->construct(wid);
+    }
+}
+
+void gmpc_plugin_preferences_destroy(gmpcPlugin *plug,GtkWidget *wid)
+{
+    if(gmpc_plugin_has_preferences(plug))
+    {
+        g_assert(plug->pref != NULL);
+        g_assert(plug->pref->destroy);
+        plug->pref->destroy(wid);
+    }
+}
+
+
+gboolean gmpc_plugin_is_internal(gmpcPlugin *plug)
+{
+    return (((plug->plugin_type)&GMPC_INTERNALL) != 0);
+}
+
+const int * gmpc_plugin_get_version(gmpcPlugin *plug)
+{
+    return (int *)plug->version;
+}
+
+int gmpc_plugin_get_type(gmpcPlugin *plug)
+{
+    return plug->plugin_type;
+}
+
+int gmpc_plugin_get_id(gmpcPlugin *plug)
+{
+    return plug->id;
+}
+
+gboolean gmpc_plugin_is_metadata(gmpcPlugin *plug)
+{
+    return (plug->metadata != NULL);
+}
+
+int gmpc_plugin_metadata_get_priority(gmpcPlugin *plug)
+{
+    if(gmpc_plugin_is_metadata(plug))
+    {
+        return plug->metadata->get_priority();
+    }
+    return 100;
+}
+
+void gmpc_plugin_metadata_set_priority(gmpcPlugin *plug, int priority)
+{
+    if(gmpc_plugin_is_metadata(plug))
+    {
+        return plug->metadata->set_priority(priority);
+    }
+}
+
+int gmpc_plugin_metadata_get_image(gmpcPlugin *plug, mpd_Song *song, MetaDataType type, char **path)
+{
+    *path = NULL;
+    if(gmpc_plugin_is_metadata(plug))
+    {
+        return plug->metadata->get_image(song, type, path);
+    }
+    return META_DATA_UNAVAILABLE;
+}
+
