@@ -1,23 +1,23 @@
-/* 
- *  Copyright (C) 1999-2008
- *
- *  Code copied from gkrellm by <Bill Wilson    billw@gkrellm.net>
- *  Modified by Qball Cow <qball@qballcow.nl>
- *
- *  This program is free software which I release under the GNU General Public
- *  License. You may redistribute and/or modify this program under the terms
- *  of that license as published by the Free Software Foundation; either
- *  version 2 of the License, or (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.  Version 2 is in the
- *  COPYRIGHT file in the top level directory of this distribution.
- * 
- *  To get a copy of the GNU General Puplic License, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+/* Gnome Music Player Client (GMPC)
+ * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Modified and borrowed from Bill Wilson <billw@gkrellm.net> (GKrellM)
+ * This projects' homepage is: http://gmpc.wikia.com/
+ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
+
 #include <config.h>
 #ifdef ENABLE_SM
 
@@ -68,6 +68,7 @@ cb_smc_shutdown_cancelled(SmcConn smc_conn, SmPointer client_data)
 cb_ice_connection_messages(IceConn ice_connection, gint source,
 		GdkInputCondition condition)
 {
+	debug_printf(DEBUG_INFO, "Ice connection");
 	IceProcessMessages(ice_connection, NULL, NULL);
 }
 
