@@ -1,12 +1,30 @@
+/* Gnome Music Player Client (GMPC)
+ * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Project homepage: http://gmpc.wikia.com/
+ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
+
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <libmpd/libmpd.h>
 #include <string.h>
 #include <config.h>
 #include "main.h"
-#include "plugin.h"
-#include "gmpc-clicklabel.h"
+#include "playlist3.h"
 #include "vala/gmpc-progress.h"
+#include "gob/gmpc-metaimage.h"
 #include "misc.h"
 
 
@@ -20,6 +38,8 @@ GtkWidget *tray_icon2_gsi = NULL;
 GtkStatusIcon 	*tray_icon2_gsi = NULL;
 #endif
 
+static gchar *current_song_checksum = NULL;
+
 static void tray_icon2_status_changed(MpdObj *mi, ChangedStatusType what, void *userdata);
 /**
  * Tooltip 
@@ -38,7 +58,7 @@ enum{
 /**
  * Preferences
  */
-static GladeXML *tray_icon2_preferences_xml = NULL;
+static GtkBuilder *tray_icon2_preferences_xml = NULL;
 void popup_timeout_changed(void);
 void popup_position_changed(GtkComboBox *om);
 void popup_enable_toggled(GtkToggleButton *but);
@@ -129,16 +149,16 @@ static void tray_icon2_populate_menu(GtkStatusIcon *gsi,guint button, guint acti
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 	g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(main_quit), NULL);
 	gtk_widget_show_all(menu);
+#ifdef EGGTRAYICON    
 	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button, activate_time);
+#else
+    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, gtk_status_icon_position_menu, gsi, button, activate_time);
+#endif
 }
 #ifndef EGGTRAYICON
 static void tray_icon2_embedded_changed(GtkStatusIcon *icon,GParamSpec *arg1, gpointer data)
 {
-    if(gtk_status_icon_is_embedded(icon))
-    {
-        /* the status icon just got embedded */
-    }
-    else
+    if(!gtk_status_icon_is_embedded(icon))
     {
         /* the widget isn't embedded anymore */
         create_playlist3();
@@ -148,10 +168,14 @@ static void tray_icon2_embedded_changed(GtkStatusIcon *icon,GParamSpec *arg1, gp
 /**
  * Initialize the tray icon 
  */
-#ifdef EGGTRAYICON
-static int tray_icon2_button_press_event(GtkWidget *tray, GdkEventButton *event, gpointer data)
+static int tray_icon2_button_press_event(gpointer tray, GdkEventButton *event, gpointer data)
 {
-    if(event->button == 1)
+    if(event->button == 2)
+    {
+        play_song();
+    }
+#ifdef EGGTRAYICON
+    else if(event->button == 1)
     {
         tray_icon2_activate(NULL, NULL);
     }
@@ -159,17 +183,19 @@ static int tray_icon2_button_press_event(GtkWidget *tray, GdkEventButton *event,
     {
         tray_icon2_populate_menu(NULL, event->button, event->time, NULL);
     }
-    else if (event->button == 2)
+#endif
+    else
     {
-       play_song(); 
+        return FALSE;
     }
+
     return TRUE;
 }
-static int tray_icon2_button_scroll_event(GtkWidget *tray, GdkEventScroll *event, gpointer data)
+
+static int tray_icon2_button_scroll_event(gpointer tray, GdkEventScroll *event, gpointer data)
 {
-    if(event->direction == GDK_SCROLL_UP)
-    {
-       volume_up(); 
+    if(event->direction == GDK_SCROLL_UP) {
+        volume_up(); 
     }else if (event->direction == GDK_SCROLL_DOWN) {
         volume_down();
     }else if (event->direction == GDK_SCROLL_LEFT) {
@@ -180,6 +206,7 @@ static int tray_icon2_button_scroll_event(GtkWidget *tray, GdkEventScroll *event
     return TRUE;
 }
 
+#ifdef EGGTRAYICON
 static int tray_icon2_button_enter_notify_event(GtkWidget *tray, GdkEventCrossing *event, gpointer data)
 {
 
@@ -207,7 +234,12 @@ static void tray_icon2_init(void)
 		/* connect the (sparse) signals */
 		g_signal_connect(G_OBJECT(tray_icon2_gsi), "popup-menu", G_CALLBACK(tray_icon2_populate_menu), NULL);
 		g_signal_connect(G_OBJECT(tray_icon2_gsi), "activate", G_CALLBACK(tray_icon2_activate), NULL);
-        g_signal_connect(G_OBJECT(tray_icon2_gsi), "notify::embedded", G_CALLBACK(tray_icon2_embedded_changed), NULL);
+		g_signal_connect(G_OBJECT(tray_icon2_gsi), "notify::embedded", G_CALLBACK(tray_icon2_embedded_changed), NULL);
+		if(gtk_check_version(2, 15, 0) == NULL)
+		{
+			g_signal_connect(G_OBJECT(tray_icon2_gsi), "button-press-event", G_CALLBACK(tray_icon2_button_press_event), NULL);
+			g_signal_connect(G_OBJECT(tray_icon2_gsi), "scroll-event", G_CALLBACK(tray_icon2_button_scroll_event), NULL);
+		}
 #endif
 	}
 }
@@ -225,6 +257,11 @@ static void tray_icon2_destroy(void)
 #endif        
         tray_icon2_gsi = NULL;
 	}
+    /* free the currnet song checksum */
+    if(current_song_checksum) {
+        g_free(current_song_checksum);
+        current_song_checksum = NULL;
+    }
 }
 /**
  * Get enabled
@@ -344,12 +381,13 @@ void tray_icon2_create_tooltip(void)
 {
 	int x=0,y=0,monitor;
 	GdkScreen *screen;
-	GtkWidget *pl3_win = glade_xml_get_widget(pl3_xml, "pl3_win");
+	GtkWidget *pl3_win = playlist3_get_window(); 
 	GtkWidget *hbox = NULL;
 	GtkWidget *vbox = NULL;
 	GtkWidget *label = NULL;
 	GtkWidget *event = NULL;
 	GtkWidget *coverimg = NULL;
+    GdkColormap *colormap;
 	mpd_Song *song = NULL;
     int tooltip_timeout = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "tooltip-timeout", 5);
 	int state = 0; 
@@ -387,8 +425,19 @@ void tray_icon2_create_tooltip(void)
 	 * 	Creat the tootlip window 
 	 */
 	tray_icon2_tooltip = gtk_window_new(GTK_WINDOW_POPUP);
+    screen = gtk_window_get_screen(GTK_WINDOW(tray_icon2_tooltip));
 
-	/* causes the border */
+
+    if (gdk_screen_is_composited(screen))
+    {
+        colormap = gdk_screen_get_rgba_colormap(screen);
+        if(colormap)
+            gtk_widget_set_colormap(tray_icon2_tooltip, colormap);
+        gtk_window_set_opacity(GTK_WINDOW(tray_icon2_tooltip), 0.9);
+    }
+
+
+    /* causes the border */
 	gtk_widget_modify_bg(GTK_WIDGET(tray_icon2_tooltip), GTK_STATE_NORMAL, &(pl3_win->style->black));
 	gtk_container_set_border_width(GTK_CONTAINER(tray_icon2_tooltip),1);
 	gtk_window_set_default_size(GTK_WINDOW(tray_icon2_tooltip), 300,-1);
@@ -410,11 +459,10 @@ void tray_icon2_create_tooltip(void)
 	 * Re-use the gmpc-metaimage widget
 	 */
 
-	coverimg = gmpc_metaimage_new(META_ALBUM_ART);
-
+	coverimg = gmpc_metaimage_new_size(META_ALBUM_ART,80);
+	gmpc_metaimage_set_squared(GMPC_METAIMAGE(coverimg), TRUE);
 	gmpc_metaimage_set_connection(GMPC_METAIMAGE(coverimg), connection);
-	gmpc_metaimage_set_size(GMPC_METAIMAGE(coverimg), 80);
-	gmpc_metaimage_set_no_cover_icon(GMPC_METAIMAGE(coverimg),"gmpc"); 
+	gmpc_metaimage_set_no_cover_icon(GMPC_METAIMAGE(coverimg),(char *)"gmpc"); 
 	/**
 	 * Force an update if mpd is playing
 	 */
@@ -495,6 +543,7 @@ void tray_icon2_create_tooltip(void)
 			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE,FALSE,0);
 		}
 		tray_icon2_tooltip_pb = (GtkWidget *) gmpc_progress_new();
+        gmpc_progress_set_hide_text(GMPC_PROGRESS(tray_icon2_tooltip_pb), TRUE);
 		/* Update the progressbar */
         gmpc_progress_set_time(GMPC_PROGRESS(tray_icon2_tooltip_pb), 
                 mpd_status_get_total_song_time(connection),
@@ -626,7 +675,7 @@ static void tray_icon2_status_changed(MpdObj *mi, ChangedStatusType what, void *
 {
 	char buffer[256];
 	mpd_Song *song = mpd_playlist_get_current_song(connection);
-	if(what&(MPD_CST_SONGID))
+	if(what&(MPD_CST_SONGID)|| what&MPD_CST_SONGPOS || what&MPD_CST_PLAYLIST)
 	{
 		/** 
 		 * If enabled by user, show the tooltip.
@@ -636,11 +685,20 @@ static void tray_icon2_status_changed(MpdObj *mi, ChangedStatusType what, void *
 		if(cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "show-tooltip", TRUE))
 		{
 			int state = mpd_player_get_state(connection);
+            gchar *new_checksum;
+            new_checksum = mpd_song_checksum(song);
 			if(state == MPD_PLAYER_PLAY || state == MPD_PLAYER_PAUSE)
 			{
-				tray_icon2_create_tooltip();
-			}
-		}
+                if(current_song_checksum == NULL || strcmp(current_song_checksum, new_checksum))
+                    tray_icon2_create_tooltip();
+            }
+            if(current_song_checksum){
+                g_free(current_song_checksum);
+                current_song_checksum = NULL;
+            }
+            current_song_checksum = new_checksum;
+        }
+
 		if(tray_icon2_gsi)
 		{
 			mpd_song_markup(buffer, 256,"[%name%: ][%title%|%shortfile%][ - %artist%]",song);
@@ -750,7 +808,7 @@ void tray_enable_toggled(GtkToggleButton *but)
 static void tray_update_settings(void)
 {
 	gtk_toggle_button_set_active((GtkToggleButton *)
-			glade_xml_get_widget(tray_icon2_preferences_xml, "ck_tray_enable"), 
+			gtk_builder_get_object(tray_icon2_preferences_xml, "ck_tray_enable"), 
 			cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "enable", DEFAULT_TRAY_ICON_ENABLE));
 }
 
@@ -768,24 +826,24 @@ void popup_position_changed(GtkComboBox *om)
 void popup_timeout_changed(void)
 {
 	cfg_set_single_value_as_int(config, TRAY_ICON2_ID, "tooltip-timeout",
-			gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(glade_xml_get_widget(tray_icon2_preferences_xml, "popup_timeout"))));
+			gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(gtk_builder_get_object(tray_icon2_preferences_xml, "popup_timeout"))));
 }
 
 static void update_popup_settings(void)
 {
 	gtk_toggle_button_set_active((GtkToggleButton *)
-			glade_xml_get_widget(tray_icon2_preferences_xml, "ck_popup_enable"),
+			gtk_builder_get_object(tray_icon2_preferences_xml, "ck_popup_enable"),
 			cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "show-tooltip", 1));
-	gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(tray_icon2_preferences_xml, "om_popup_position")),
+	gtk_combo_box_set_active(GTK_COMBO_BOX(gtk_builder_get_object(tray_icon2_preferences_xml, "om_popup_position")),
 			cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "tooltip-position", 0));
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(tray_icon2_preferences_xml, "popup_timeout")),
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(gtk_builder_get_object(tray_icon2_preferences_xml, "popup_timeout")),
 			cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "tooltip-timeout", 5));
 }
 
 static void tray_icon2_preferences_destroy(GtkWidget *container)
 {
 	if(tray_icon2_preferences_xml) {
-		GtkWidget *vbox = glade_xml_get_widget(tray_icon2_preferences_xml, "tray-pref-vbox");
+		GtkWidget *vbox =(GtkWidget *) gtk_builder_get_object(tray_icon2_preferences_xml, "tray-pref-vbox");
 		gtk_container_remove(GTK_CONTAINER(container),vbox);
 		g_object_unref(tray_icon2_preferences_xml);
 		tray_icon2_preferences_xml = NULL;
@@ -798,17 +856,19 @@ void tray_icon2_preferences_pm_combo_changed(GtkComboBox *cm, gpointer data)
 }
 static void tray_icon2_preferences_construct(GtkWidget *container)
 {
-	gchar *path = gmpc_get_full_glade_path("gmpc.glade");
-	tray_icon2_preferences_xml = glade_xml_new(path, "tray-pref-vbox",NULL);
+	gchar *path = gmpc_get_full_glade_path("preferences-trayicon.ui");
+	tray_icon2_preferences_xml = gtk_builder_new();
+    gtk_builder_add_from_file(tray_icon2_preferences_xml, path, NULL);
+    q_free(path);
 
 	if(tray_icon2_preferences_xml) {
-		GtkWidget *vbox = glade_xml_get_widget(tray_icon2_preferences_xml, "tray-pref-vbox");
+		GtkWidget *vbox =(GtkWidget *) gtk_builder_get_object(tray_icon2_preferences_xml, "tray-pref-vbox");
 		gtk_container_add(GTK_CONTAINER(container),vbox);
 		tray_update_settings();
 		update_popup_settings();
-		glade_xml_signal_autoconnect(tray_icon2_preferences_xml);
+        gtk_builder_connect_signals(tray_icon2_preferences_xml, NULL);
 	}
-    gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget(tray_icon2_preferences_xml, "pm-combo")),
+    gtk_combo_box_set_active(GTK_COMBO_BOX(gtk_builder_get_object(tray_icon2_preferences_xml, "pm-combo")),
         cfg_get_single_value_as_int_with_default(config, "Default","min-error-level", ERROR_INFO));
 }
 gmpcPrefPlugin tray_icon2_preferences = {
