@@ -1,3 +1,21 @@
+/* Gnome Music Player Client (GMPC)
+ * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Project homepage: http://gmpc.wikia.com/
+ 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
 
 #ifndef __GMPC_PROGRESS_H__
 #define __GMPC_PROGRESS_H__
@@ -21,22 +39,21 @@ typedef struct _GmpcProgressClass GmpcProgressClass;
 typedef struct _GmpcProgressPrivate GmpcProgressPrivate;
 
 struct _GmpcProgress {
-	GtkEventBox parent_instance;
+	GtkHBox parent_instance;
 	GmpcProgressPrivate * priv;
 	gboolean _hide_text;
 };
 
 struct _GmpcProgressClass {
-	GtkEventBoxClass parent_class;
+	GtkHBoxClass parent_class;
 };
 
 
 void gmpc_progress_set_time (GmpcProgress* self, guint total, guint current);
+GmpcProgress* gmpc_progress_construct (GType object_type);
 GmpcProgress* gmpc_progress_new (void);
 gboolean gmpc_progress_get_hide_text (GmpcProgress* self);
 void gmpc_progress_set_hide_text (GmpcProgress* self, gboolean value);
-gboolean gmpc_progress_get_do_countdown (GmpcProgress* self);
-void gmpc_progress_set_do_countdown (GmpcProgress* self, gboolean value);
 GType gmpc_progress_get_type (void);
 
 
